with
  text_io,
  ada.integer_text_io,
  ada.long_float_text_io,
  ada.numerics,
  ada.numerics.long_elementary_functions;

procedure cone is

  type vec3 is array (1 .. 3) of long_float;

  n1,n2,nn,ne          : integer;
  r1,r2,dr,h,thickness : long_float;

  procedure scan (io : in text_io.file_type) is

    ascii : character;

  begin
    loop
      text_io.get(io,ascii);
      text_io.skip_line(io);
      exit when ascii = ':';
    end loop;
  end scan;

  function node (i,j : in integer) return integer is

  begin
    return 3 * (i * (i - 1) - n1 * (n1 - 1)) + j;
  end node;

  procedure vertex (i,j : in integer;
                    x   : out vec3) is

    use
      ada.numerics,
      ada.numerics.long_elementary_functions;

    r,t : long_float;

  begin
    r := dr * long_float(i);

    if 2 * (j - 1) = 0 then
      x(1) := r;
      x(2) := 0.0;
    elsif 2 * (j - 1) = 3 * i then
      x(1) := 0.0;
      x(2) := r;
    elsif 2 * (j - 1) = 6 * i then
      x(1) := -r;
      x(2) := 0.0;
    elsif 2 * (j - 1) = 9 * i then
      x(1) := 0.0;
      x(2) := -r;
    else
      t := pi / 3.0 / long_float(i) * long_float(j - 1);
      x(1) := r * cos(t);
      x(2) := r * sin(t);
    end if;

    x(3) := h / long_float(n2 - n1) * long_float(n2 - i);
  end vertex;

  function firstj (i,k : in integer) return integer is

  begin
    return (k - 1) * i + 1;
  end firstj;

  function nextj (i,j : in integer) return integer is

  begin
    return j mod (6 * i) + 1;
  end nextj;

  procedure initia is

    use
      text_io,
      ada.integer_text_io,
      ada.long_float_text_io;

    io : file_type;

  begin
    open(io, in_file, "cone.txt");

    -- The circumference with radius r1 is divided in 6 * n1 parts.
    -- The circumference with radius r2 is divided in 6 * n2 parts.
    -- The height h is divided in (n2 - n1) parts.

    -- r2 > r1 > 0.0
    -- n2 > n1 > 1

    scan(io);
    get(io,r1);
    get(io,n1);
    get(io,r2);
    get(io,n2);
    get(io,h);
    get(io,thickness);
    skip_line(io);

    close(io);

    -- dr
    dr := (r2 - r1) / long_float(n2 - n1);

    -- nn, ne
    nn := 3 * (n1 + n2) * (n2 - n1 + 1);
    ne := 6 * (n2 - n1) * (n2 + n1);
  end initia;

  procedure input is

    use
      text_io,
      ada.integer_text_io,
      ada.long_float_text_io;

    maxls,maxminor,maxlist,maxmajor : integer;
    lstol,maxnlag,maxncon,mu        : long_float;
    e,j1,j2,j3,glob                 : integer;
    x                               : vec3;
    io                              : file_type;

  begin
    -- solution parameters
    lstol := 1.0E-01;
    maxls := 20;
    maxminor := 30 * nn;
    maxlist := 3;
    maxnlag := 1.0E-05;
    mu := 0.0;
    maxncon := 0.0;
    maxmajor := 0;

    create(io, out_file, "input.txt");

    put(io,"        lstol     maxls  maxminor   maxlist                mu           maxnlag           maxncon  maxmajor");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,lstol,2,6,3);
    put(io,"     ");
    put(io,maxls,5);
    put(io,"     ");
    put(io,maxminor,5);
    put(io,"     ");
    put(io,maxlist,5);
    put(io,"     ");
    put(io,mu,2,6,3);
    put(io,"     ");
    put(io,maxnlag,2,6,3);
    put(io,"     ");
    put(io,maxncon,2,6,3);
    put(io,"     ");
    put(io,maxmajor,5);
    new_line(io);

    -- node coordinates
    new_line(io);
    new_line(io);
    put(io," node           coord x           coord y           coord z");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,nn,5);
    new_line(io);

    new_line(io);

    for i in n1 .. n2 loop
      for j in 1 .. 6 * i loop
        vertex(i,j,x);
        put(io,node(i,j),5);
        put(io,"     ");
        put(io,x(1),2,6,3);
        put(io,"     ");
        put(io,x(2),2,6,3);
        put(io,"     ");
        put(io,x(3),2,6,3);
        new_line(io);
      end loop;
    end loop;

    -- element 11
    new_line(io);
    new_line(io);
    put(io,"elem1      node      node              area");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,"    0");
    new_line(io);

    -- element 12
    new_line(io);
    new_line(io);
    put(io,"elem2      node      node");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,"    0");
    new_line(io);

    -- element 21
    new_line(io);
    new_line(io);
    put(io," elem      node      node      node         thickness");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,ne,5);
    new_line(io);

    e := 0;
    new_line(io);

    for k in 1 .. 6 loop
      for i in n1 .. n2 - 1 loop
        j1 := firstj(i,k);
        j2 := firstj(i + 1,k);
        for s in 1 .. i + 1 loop
          j3 := nextj(i + 1,j2);

          e := e + 1;
          put(io,e,5);
          put(io,"     ");
          put(io,node(i,j1),5);
          put(io,"     ");
          put(io,node(i + 1,j2),5);
          put(io,"     ");
          put(io,node(i + 1,j3),5);
          put(io,"     ");
          put(io,thickness,2,6,3);
          new_line(io);

          j1 := nextj(i,j1);
          j2 := j3;
        end loop;
      end loop;
    end loop;

    for k in 1 .. 6 loop
      for i in n1 .. n2 - 1 loop
        j1 := firstj(i,k);
        j2 := firstj(i + 1,k);
        j2 := nextj(i + 1,j2);
        for s in 1 .. i loop
          j3 := nextj(i,j1);

          e := e + 1;
          put(io,e,5);
          put(io,"     ");
          put(io,node(i,j1),5);
          put(io,"     ");
          put(io,node(i + 1,j2),5);
          put(io,"     ");
          put(io,node(i,j3),5);
          put(io,"     ");
          put(io,thickness,2,6,3);
          new_line(io);

          j1 := j3;
          j2 := nextj(i + 1,j2);
        end loop;
      end loop;
    end loop;

    -- element 22
    new_line(io);
    new_line(io);
    put(io,"elem2      node      node      node");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,"    0");
    new_line(io);

    -- element 31
    new_line(io);
    new_line(io);
    put(io,"elem1      node      node      node      node");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,"    0");
    new_line(io);

    -- element 32
    new_line(io);
    new_line(io);
    put(io,"elem2      node      node      node      node");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,"    0");
    new_line(io);

    -- known displacements
    new_line(io);
    new_line(io);
    put(io," node      axis             displ");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,18 * (n1 + n2),5);
    new_line(io);

    new_line(io);

    -- top
    for j in 1 .. 6 * n1 loop
      glob := node(n1,j);

      put(io,glob,5);
      put(io,"         1      0.000000E+00");
      new_line(io);

      put(io,glob,5);
      put(io,"         2      0.000000E+00");
      new_line(io);

      put(io,glob,5);
      put(io,"         3      0.000000E+00");
      new_line(io);
    end loop;

    -- bottom
    for j in 1 .. 6 * n2 loop
      glob := node(n2,j);

      put(io,glob,5);
      put(io,"         1      0.000000E+00");
      new_line(io);

      put(io,glob,5);
      put(io,"         2      0.000000E+00");
      new_line(io);

      put(io,glob,5);
      put(io,"         3      0.000000E+00");
      new_line(io);
    end loop;

    close(io);
  end input;

  procedure autocad is

    use
      text_io,
      ada.long_float_text_io;

    j1,j2,j3 : integer;
    x        : vec3;
    io       : file_type;

  begin
    create(io, out_file, "cone.scr");

    -- set running object snap mode to NONE
    put(io,"osmode 0");
    new_line(io);

    -- element 21
    put(io,"-color");
    new_line(io);
    put(io,"green");
    new_line(io);

    put(io,"layer make Initial_21 ");
    new_line(io);

    for k in 1 .. 6 loop
      for i in n1 .. n2 - 1 loop
        j1 := firstj(i,k);
        j2 := firstj(i + 1,k);
        for s in 1 .. i + 1 loop
          j3 := nextj(i + 1,j2);

          put(io,"3dface ");

          vertex(i,j1,x);
          put(io,x(1),1,8,3);
          put(io,",");
          put(io,x(2),1,8,3);
          put(io,",");
          put(io,x(3),1,8,3);
          put(io," ");

          vertex(i + 1,j2,x);
          put(io,x(1),1,8,3);
          put(io,",");
          put(io,x(2),1,8,3);
          put(io,",");
          put(io,x(3),1,8,3);
          put(io," ");

          vertex(i + 1,j3,x);
          put(io,x(1),1,8,3);
          put(io,",");
          put(io,x(2),1,8,3);
          put(io,",");
          put(io,x(3),1,8,3);
          put(io," ");

          put(io," ");
          new_line(io);

          j1 := nextj(i,j1);
          j2 := j3;
        end loop;
      end loop;
    end loop;

    for k in 1 .. 6 loop
      for i in n1 .. n2 - 1 loop
        j1 := firstj(i,k);
        j2 := firstj(i + 1,k);
        j2 := nextj(i + 1,j2);
        for s in 1 .. i loop
          j3 := nextj(i,j1);

          put(io,"3dface ");

          vertex(i,j1,x);
          put(io,x(1),1,8,3);
          put(io,",");
          put(io,x(2),1,8,3);
          put(io,",");
          put(io,x(3),1,8,3);
          put(io," ");

          vertex(i + 1,j2,x);
          put(io,x(1),1,8,3);
          put(io,",");
          put(io,x(2),1,8,3);
          put(io,",");
          put(io,x(3),1,8,3);
          put(io," ");

          vertex(i,j3,x);
          put(io,x(1),1,8,3);
          put(io,",");
          put(io,x(2),1,8,3);
          put(io,",");
          put(io,x(3),1,8,3);
          put(io," ");

          put(io," ");
          new_line(io);

          j1 := j3;
          j2 := nextj(i + 1,j2);
        end loop;
      end loop;
    end loop;

    put(io,"-color");
    new_line(io);
    put(io,"white");
    new_line(io);

    put(io,"layer set 0 ");
    new_line(io);

    put(io,"zoom ext ext");
    new_line(io);

    close(io);
  end autocad;

begin
  initia;
  input;
  autocad;
end cone;
