with
  text_io,
  ada.integer_text_io,
  ada.long_float_text_io,
  ada.numerics,
  ada.numerics.long_elementary_functions;

procedure cprism is

  type vec3 is array (1 .. 3) of long_float;

  nt,nc      : integer;
  rt,rc,area : long_float;

  procedure scan (io : in text_io.file_type) is

    ascii : character;

  begin
    loop
      text_io.get(io,ascii);
      text_io.skip_line(io);
      exit when ascii = ':';
    end loop;
  end scan;

  function spot (i,j : in integer) return integer is

  begin
    return nc * (i mod nt) + j mod nc + 1;
  end spot;

  procedure vertex (i,j : in integer;
                    x   : out vec3) is

    use
      ada.numerics,
      ada.numerics.long_elementary_functions;

    anglet,anglec : long_float;

  begin
    anglet := (pi + pi) / long_float(nt) * long_float(i);
    anglec := (pi + pi) / long_float(nc) * long_float(j);

    x(1) := (rt + rc * cos(anglec)) * cos(anglet);
    x(2) := (rt + rc * cos(anglec)) * sin(anglet);
    x(3) := rc * sin(anglec);
  end vertex;

  procedure initia is

    use
      text_io,
      ada.integer_text_io,
      ada.long_float_text_io;

    io : file_type;

  begin
    open(io, in_file, "cprism.txt");

    -- nt : number of sides of the inscribed polygonal torus
    -- nc : number of sides of the inscribed polygon
    -- rt : radius of the polygonal torus
    -- rc : radius of the polygon
    scan(io);
    get(io,nt);
    get(io,nc);
    get(io,rt);
    get(io,rc);
    get(io,area);
    skip_line(io);

    close(io);
  end initia;

  procedure input is

    use
      text_io,
      ada.integer_text_io,
      ada.long_float_text_io;

    maxls,maxminor,maxlist,maxmajor : integer;
    lstol,maxnlag,maxncon,mu        : long_float;
    e                               : integer;
    x                               : vec3;
    io                              : file_type;

  begin
    -- solution parameters
    lstol := 1.0E-01;
    maxls := 20;
    maxminor := 30 * nt * nc;
    maxlist := 3;
    mu := 1.0E-03;
    maxnlag := 1.0E-05;
    maxncon := 1.0E-06;
    maxmajor := 20 * nc * nt;

    create(io, out_file, "input.txt");

    put(io,"        lstol     maxls  maxminor   maxlist                mu           maxnlag           maxncon  maxmajor");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,lstol,2,6,3);
    put(io,"     ");
    put(io,maxls,5);
    put(io,"     ");
    put(io,maxminor,5);
    put(io,"     ");
    put(io,maxlist,5);
    put(io,"     ");
    put(io,mu,2,6,3);
    put(io,"     ");
    put(io,maxnlag,2,6,3);
    put(io,"     ");
    put(io,maxncon,2,6,3);
    put(io,"     ");
    put(io,maxmajor,5);
    new_line(io);

    -- node coordinates
    new_line(io);
    new_line(io);
    put(io," node           coord x           coord y           coord z");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,nt * nc,5);
    new_line(io);

    new_line(io);
    for i in 0 .. nt - 1 loop
      for j in 0 .. nc - 1 loop
        vertex(i,j,x);
        put(io,spot(i,j),5);
        put(io,"     ");
        put(io,x(1),2,6,3);
        put(io,"     ");
        put(io,x(2),2,6,3);
        put(io,"     ");
        put(io,x(3),2,6,3);
        new_line(io);
      end loop;
    end loop;

    -- element 11
    new_line(io);
    new_line(io);
    put(io,"elem1      node      node              area");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,nc * nt,5);
    new_line(io);

    new_line(io);
    e := 0;

    -- diagonal
    for i in 0 .. nt - 1 loop
      if i mod 2 = 0 then
        for j in 0 .. nc - 1 loop
          e := e + 1;

          put(io,e,5);
          put(io,"     ");
          put(io,spot(i,j + 1),5);
          put(io,"     ");
          put(io,spot(i + 1,j),5);
          put(io,"     ");
          put(io,area,2,6,3);
          new_line(io);
        end loop;
      else
        for j in 0 .. nc - 1 loop
          e := e + 1;

          put(io,e,5);
          put(io,"     ");
          put(io,spot(i,j),5);
          put(io,"     ");
          put(io,spot(i + 1,j + 1),5);
          put(io,"     ");
          put(io,area,2,6,3);
          new_line(io);
        end loop;
      end if;
    end loop;

    -- element 12
    new_line(io);
    new_line(io);
    put(io,"elem2      node      node");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,2 * nc * nt,5);
    new_line(io);

    new_line(io);
    e := 0;

    -- polygon
    for i in 0 .. nt - 1 loop
      for j in 0 .. nc - 1 loop
        e := e + 1;

        put(io,e,5);
        put(io,"     ");
        put(io,spot(i,j),5);
        put(io,"     ");
        put(io,spot(i,j + 1),5);
        new_line(io);
      end loop;
    end loop;

    -- vertical
    for i in 0 .. nt - 1 loop
      for j in 0 .. nc - 1 loop
        e := e + 1;

        put(io,e,5);
        put(io,"     ");
        put(io,spot(i,j),5);
        put(io,"     ");
        put(io,spot(i + 1,j),5);
        new_line(io);
      end loop;
    end loop;

    -- element 21
    new_line(io);
    new_line(io);
    put(io,"elem1      node      node      node         thickness");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,"    0");
    new_line(io);

    -- element 22
    new_line(io);
    new_line(io);
    put(io,"elem2      node      node      node");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,"    0");
    new_line(io);

    -- element 31
    new_line(io);
    new_line(io);
    put(io,"elem1      node      node      node      node");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,"    0");
    new_line(io);

    -- element 32
    new_line(io);
    new_line(io);
    put(io,"elem2      node      node      node      node");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,"    0");
    new_line(io);

    -- known displacements
    new_line(io);
    new_line(io);
    put(io," node      axis             displ");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,"    0");
    new_line(io);

    close(io);
  end input;

  procedure autocad is

    use
      text_io,
      ada.integer_text_io,
      ada.long_float_text_io;

    x  : vec3;
    io : file_type;

  begin
    create(io, out_file, "cprism.scr");

    -- set running object snap mode to NONE
    put(io,"osmode 0");
    new_line(io);

    -- element 11
    put(io,"-color");
    new_line(io);
    put(io,"red");
    new_line(io);

    put(io,"layer make Initial_11 ");
    new_line(io);

    -- diagonal
    for i in 0 .. nt - 1 loop
      if i mod 2 = 0 then
        for j in 0 .. nc - 1 loop
          put(io,"3dpoly ");

          vertex(i,j + 1,x);
          put(io,x(1),1,8,3);
          put(io,",");
          put(io,x(2),1,8,3);
          put(io,",");
          put(io,x(3),1,8,3);
          put(io," ");

          vertex(i + 1,j,x);
          put(io,x(1),1,8,3);
          put(io,",");
          put(io,x(2),1,8,3);
          put(io,",");
          put(io,x(3),1,8,3);
          put(io," ");

          new_line(io);
        end loop;
      else
        for j in 0 .. nc - 1 loop
          put(io,"3dpoly ");

          vertex(i,j,x);
          put(io,x(1),1,8,3);
          put(io,",");
          put(io,x(2),1,8,3);
          put(io,",");
          put(io,x(3),1,8,3);
          put(io," ");

          vertex(i + 1,j + 1,x);
          put(io,x(1),1,8,3);
          put(io,",");
          put(io,x(2),1,8,3);
          put(io,",");
          put(io,x(3),1,8,3);
          put(io," ");

          new_line(io);
        end loop;
      end if;
    end loop;

    -- element 12
    put(io,"-color");
    new_line(io);
    put(io,"cyan");
    new_line(io);

    put(io,"layer make Initial_12 ");
    new_line(io);

    -- polygon
    for i in 0 .. nt - 1 loop
      for j in 0 .. nc - 1 loop
        put(io,"3dpoly ");

        vertex(i,j,x);
        put(io,x(1),1,8,3);
        put(io,",");
        put(io,x(2),1,8,3);
        put(io,",");
        put(io,x(3),1,8,3);
        put(io," ");

        vertex(i,j + 1,x);
        put(io,x(1),1,8,3);
        put(io,",");
        put(io,x(2),1,8,3);
        put(io,",");
        put(io,x(3),1,8,3);
        put(io," ");

        new_line(io);
      end loop;
    end loop;

    -- vertical
    for i in 0 .. nt - 1 loop
      for j in 0 .. nc - 1 loop
        put(io,"3dpoly ");

        vertex(i,j,x);
        put(io,x(1),1,8,3);
        put(io,",");
        put(io,x(2),1,8,3);
        put(io,",");
        put(io,x(3),1,8,3);
        put(io," ");

        vertex(i + 1,j,x);
        put(io,x(1),1,8,3);
        put(io,",");
        put(io,x(2),1,8,3);
        put(io,",");
        put(io,x(3),1,8,3);
        put(io," ");

        new_line(io);
      end loop;
    end loop;

    put(io,"-color");
    new_line(io);
    put(io,"white");
    new_line(io);

    put(io,"layer set 0 ");
    new_line(io);

    put(io,"zoom ext ext");
    new_line(io);

    close(io);
  end autocad;

begin
  initia;
  input;
  autocad;
end cprism;
