with
  text_io,
  ada.integer_text_io,
  ada.long_float_text_io,
  ada.numerics,
  ada.numerics.long_elementary_functions;

procedure cylinder3 is

  type vec3 is array (1 .. 3) of long_float;

  ni,nj,ns,type1,type2,twist : integer;
  radius,height              : long_float;
  area,thickness             : long_float;

  procedure scan (io : in text_io.file_type) is

    ascii : character;

  begin
    loop
      text_io.get(io,ascii);
      text_io.skip_line(io);
      exit when ascii = ':';
    end loop;
  end scan;

  function node (i,j : in integer) return integer is

  begin
    return ni * j + (i mod ni) + 1;
  end node;

  procedure vertex (i,j : in integer;
                    x   : out vec3) is

    use
      ada.numerics,
      ada.numerics.long_elementary_functions;

    t : long_float;

  begin
    t := 2.0 * pi / long_float(ni) * long_float(i mod ni);
    x(1) := radius * cos(t);
    x(2) := radius * sin(t);
    x(3) := (long_float(j) / long_float(nj) - 0.5) * height;
  end vertex;

  procedure initia is

    use
      text_io,
      ada.integer_text_io,
      ada.long_float_text_io;

    io : file_type;

  begin
    open(io, in_file, "cylinder3.txt");

    -- The circumference is divided in ni parts
    -- The height is divided in nj parts
    -- The numbers ni and nj must be even
    -- The diagonals are placed skipping skipping (ns - 1) divisions
    -- The number ni must be divisible by the number ns

    scan(io);
    get(io,radius);
    get(io,ni);
    get(io,height);
    get(io,nj);
    get(io,ns);
    skip_line(io);

    -- element 1
    scan(io);
    get(io,area);
    get(io,type1);
    get(io,twist);
    skip_line(io);

    -- element 2
    scan(io);
    get(io,thickness);
    get(io,type2);
    skip_line(io);

    close(io);
  end initia;

  procedure input is

    use
      text_io,
      ada.integer_text_io,
      ada.long_float_text_io;

    maxls,maxminor,maxlist,maxmajor : integer;
    lstol,maxnlag,maxncon,mu        : long_float;
    e,ne,nk,i,j,glob                : integer;
    x                               : vec3;
    io                              : file_type;

    procedure putelem11 (e : in integer; glob1 : in integer; glob2 : in integer) is

    begin
      put(io,e,5);
      put(io,"     ");
      put(io,glob1,5);
      put(io,"     ");
      put(io,glob2,5);
      put(io,"     ");
      put(io,area,2,6,3);
      new_line(io);
    end putelem11;

    procedure putelem12 (e : in integer; glob1 : in integer; glob2 : in integer) is

    begin
      put(io,e,5);
      put(io,"     ");
      put(io,glob1,5);
      put(io,"     ");
      put(io,glob2,5);
      new_line(io);
    end putelem12;

    procedure putelem21 (e : in integer; glob1 : in integer; glob2 : in integer; glob3 : in integer) is

    begin
      put(io,e,5);
      put(io,"     ");
      put(io,glob1,5);
      put(io,"     ");
      put(io,glob2,5);
      put(io,"     ");
      put(io,glob3,5);
      put(io,"     ");
      put(io,thickness,2,6,3);
      new_line(io);
    end putelem21;

    procedure putelem22 (e : in integer; glob1 : in integer; glob2 : in integer; glob3 : in integer) is

    begin
      put(io,e,5);
      put(io,"     ");
      put(io,glob1,5);
      put(io,"     ");
      put(io,glob2,5);
      put(io,"     ");
      put(io,glob3,5);
      new_line(io);
    end putelem22;

  begin
    -- solution parameters
    lstol := 1.0E-01;
    maxls := 20;
    maxminor := 30 * ni * (nj + 1);
    maxlist := 3;
    maxnlag := 1.0E-03;
    mu := 1.0E-03;
    maxncon := 1.0E-03;
    maxmajor := 0;
    if type1 = 2 then
      maxmajor := maxmajor + ni / ns;
    end if;
    if type2 = 2 then
      maxmajor := maxmajor + 2 * ni * nj;
    end if;
    maxmajor := 10 * maxmajor;

    create(io, out_file, "input.txt");

    put(io,"        lstol     maxls  maxminor   maxlist                mu           maxnlag           maxncon  maxmajor");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,lstol,2,6,3);
    put(io,"     ");
    put(io,maxls,5);
    put(io,"     ");
    put(io,maxminor,5);
    put(io,"     ");
    put(io,maxlist,5);
    put(io,"     ");
    put(io,mu,2,6,3);
    put(io,"     ");
    put(io,maxnlag,2,6,3);
    put(io,"     ");
    put(io,maxncon,2,6,3);
    put(io,"     ");
    put(io,maxmajor,5);
    new_line(io);

    -- node coordinates
    new_line(io);
    new_line(io);
    put(io," node           coord x           coord y           coord z");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,ni * (nj + 1),5);
    new_line(io);

    new_line(io);
    for j in 0 .. nj loop
      for i in 0 .. ni - 1 loop
        vertex(i,j,x);
        put(io,node(i,j),5);
        put(io,"     ");
        put(io,x(1),2,6,3);
        put(io,"     ");
        put(io,x(2),2,6,3);
        put(io,"     ");
        put(io,x(3),2,6,3);
        new_line(io);
      end loop;
    end loop;

    -- element 11
    if type1 = 1 then
      ne := ni / ns;
    else
      ne := 0;
    end if;

    new_line(io);
    new_line(io);
    put(io,"elem1      node      node              area");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,ne,5);
    new_line(io);

    -- diagonal
    if type1 = 1 then
      new_line(io);
      e := 0;

    case twist is
      when -1 =>
        for i in 1 .. ni / ns loop
          e := e + 1;
          putelem11(e,node(i * ns,0),node((i - 1) * ns,nj));
        end loop;

      when 0 =>
        for i in 0 .. ni / ns - 1 loop
          e := e + 1;
          putelem11(e,node(i * ns,0),node(i * ns,nj));
        end loop;

      when 1 =>
        for i in 1 .. ni / ns loop
          e := e + 1;
          putelem11(e,node((i - 1) * ns,0),node(i * ns,nj));
        end loop;

      when others =>
        null;
    end case;
    end if;

    -- element 12
    if type1 = 2 then
      ne := ni / ns;
    else
      ne := 0;
    end if;

    new_line(io);
    new_line(io);
    put(io,"elem2      node      node");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,ne,5);
    new_line(io);

    -- diagonal
    if type1 = 2 then
      new_line(io);
      e := 0;

    case twist is
      when -1 =>
        for i in 1 .. ni / ns loop
          e := e + 1;
          putelem12(e,node(i * ns,0),node((i - 1) * ns,nj));
        end loop;

      when 0 =>
        for i in 0 .. ni / ns - 1 loop
          e := e + 1;
          putelem12(e,node(i * ns,0),node(i * ns,nj));
        end loop;

      when 1 =>
        for i in 1 .. ni / ns loop
          e := e + 1;
          putelem12(e,node((i - 1) * ns,0),node(i * ns,nj));
        end loop;

      when others =>
        null;
      end case;
    end if;

    -- element 21
    if type2 = 1 then
      ne := 2 * ni * nj;
    else
      ne := 0;
    end if;

    new_line(io);
    new_line(io);
    put(io,"elem1      node      node      node         thickness");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,ne,5);
    new_line(io);

    if type2 = 1 then
      new_line(io);
      e := 0;

      j := 1;
      while j < nj loop

        i := 1;
        while i < ni loop
          e := e + 1;
          putelem21(e,node(i,j),node(i + 1,j),node(i + 1,j + 1));
          e := e + 1;
          putelem21(e,node(i,j),node(i + 1,j + 1),node(i,j + 1));
          e := e + 1;
          putelem21(e,node(i,j),node(i,j + 1),node(i - 1,j + 1));
          e := e + 1;
          putelem21(e,node(i,j),node(i - 1,j + 1),node(i - 1,j));
          e := e + 1;
          putelem21(e,node(i,j),node(i - 1,j),node(i - 1,j - 1));
          e := e + 1;
          putelem21(e,node(i,j),node(i - 1,j - 1),node(i,j - 1));
          e := e + 1;
          putelem21(e,node(i,j),node(i,j - 1),node(i + 1,j - 1));
          e := e + 1;
          putelem21(e,node(i,j),node(i + 1,j - 1),node(i + 1,j));

          i := i + 2;
        end loop;

        j := j + 2;
      end loop;
    end if;

    -- element 22
    if type2 = 2 then
      ne := 2 * ni * nj;
    else
      ne := 0;
    end if;

    new_line(io);
    new_line(io);
    put(io,"elem2      node      node      node");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,ne,5);
    new_line(io);

    if type2 = 2 then
      new_line(io);
      e := 0;

      j := 1;
      while j < nj loop

        i := 1;
        while i < ni loop
          e := e + 1;
          putelem22(e,node(i,j),node(i + 1,j),node(i + 1,j + 1));
          e := e + 1;
          putelem22(e,node(i,j),node(i + 1,j + 1),node(i,j + 1));
          e := e + 1;
          putelem22(e,node(i,j),node(i,j + 1),node(i - 1,j + 1));
          e := e + 1;
          putelem22(e,node(i,j),node(i - 1,j + 1),node(i - 1,j));
          e := e + 1;
          putelem22(e,node(i,j),node(i - 1,j),node(i - 1,j - 1));
          e := e + 1;
          putelem22(e,node(i,j),node(i - 1,j - 1),node(i,j - 1));
          e := e + 1;
          putelem22(e,node(i,j),node(i,j - 1),node(i + 1,j - 1));
          e := e + 1;
          putelem22(e,node(i,j),node(i + 1,j - 1),node(i + 1,j));

          i := i + 2;
        end loop;

        j := j + 2;
      end loop;
    end if;

    -- element 31
    new_line(io);
    new_line(io);
    put(io,"elem1      node      node      node      node");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,"    0");
    new_line(io);

    -- element 32
    new_line(io);
    new_line(io);
    put(io,"elem2      node      node      node      node");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,"    0");
    new_line(io);

    -- known displacements
    if type1 = 0 then
      nk := 6 * ni;
    else
      nk := 0;
    end if;

    new_line(io);
    new_line(io);
    put(io," node      axis             displ");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,nk,5);
    new_line(io);

    if type1 = 0 then
      new_line(io);

      -- bottom
      for i in 0 .. ni - 1 loop
        glob := node(i,0);

        for axis in 1 .. 3 loop
          put(io,glob,5);
          put(io,"     ");
          put(io,axis,5);
          put(io,"      0.000000E+00");
          new_line(io);
        end loop;
      end loop;

      -- top
      for i in 0 .. ni - 1 loop
        glob := node(i,nj);

        for axis in 1 .. 3 loop
          put(io,glob,5);
          put(io,"     ");
          put(io,axis,5);
          put(io,"      0.000000E+00");
          new_line(io);
        end loop;
      end loop;
    end if;

    close(io);
  end input;

  procedure autocad is

    use
      text_io,
      ada.integer_text_io,
      ada.long_float_text_io;

    i,j : integer;
    x   : vec3;
    io  : file_type;

    procedure putelem1 (i1 : in integer; j1 : in integer; i2 : in integer; j2 : in integer) is

    begin
      put(io,"3dpoly ");

      vertex(i1,j1,x);
      put(io,x(1),1,8,3);
      put(io,",");
      put(io,x(2),1,8,3);
      put(io,",");
      put(io,x(3),1,8,3);
      put(io," ");

      vertex(i2,j2,x);
      put(io,x(1),1,8,3);
      put(io,",");
      put(io,x(2),1,8,3);
      put(io,",");
      put(io,x(3),1,8,3);
      put(io," ");

      new_line(io);
    end putelem1;

    procedure putelem2 (i1,j1,i2,j2,i3,j3 : in integer) is

    begin
      put(io,"3dface ");

      vertex(i1,j1,x);
      put(io,x(1),1,8,3);
      put(io,",");
      put(io,x(2),1,8,3);
      put(io,",");
      put(io,x(3),1,8,3);
      put(io," ");

      vertex(i2,j2,x);
      put(io,x(1),1,8,3);
      put(io,",");
      put(io,x(2),1,8,3);
      put(io,",");
      put(io,x(3),1,8,3);
      put(io," ");

      vertex(i3,j3,x);
      put(io,x(1),1,8,3);
      put(io,",");
      put(io,x(2),1,8,3);
      put(io,",");
      put(io,x(3),1,8,3);
      put(io," ");

      put(io," ");
      new_line(io);
    end putelem2;

  begin
    create(io, out_file, "cylinder3.scr");

    -- set running object snap mode to NONE
    put(io,"osmode 0");
    new_line(io);

    -- element 21
    if type2 = 1 then
      put(io,"layer make Initial_21 ");
      new_line(io);
      put(io,"-color");
      new_line(io);
      put(io,"green");
      new_line(io);

      j := 1;
      while j < nj loop

        i := 1;
        while i < ni loop
          putelem2(i,j,i + 1,j,i + 1,j + 1);
          putelem2(i,j,i + 1,j + 1,i,j + 1);
          putelem2(i,j,i,j + 1,i - 1,j + 1);
          putelem2(i,j,i - 1,j + 1,i - 1,j);
          putelem2(i,j,i - 1,j,i - 1,j - 1);
          putelem2(i,j,i - 1,j - 1,i,j - 1);
          putelem2(i,j,i,j - 1,i + 1,j - 1);
          putelem2(i,j,i + 1,j - 1,i + 1,j);

          i := i + 2;
        end loop;

        j := j + 2;
      end loop;
    end if;

    -- element 22
    if type2 = 2 then
      put(io,"layer make Initial_21 ");
      new_line(io);
      put(io,"-color");
      new_line(io);
      put(io,"magenta");
      new_line(io);

      j := 1;
      while j < nj loop

        i := 1;
        while i < ni loop
          putelem2(i,j,i + 1,j,i + 1,j + 1);
          putelem2(i,j,i + 1,j + 1,i,j + 1);
          putelem2(i,j,i,j + 1,i - 1,j + 1);
          putelem2(i,j,i - 1,j + 1,i - 1,j);
          putelem2(i,j,i - 1,j,i - 1,j - 1);
          putelem2(i,j,i - 1,j - 1,i,j - 1);
          putelem2(i,j,i,j - 1,i + 1,j - 1);
          putelem2(i,j,i + 1,j - 1,i + 1,j);

          i := i + 2;
        end loop;

        j := j + 2;
      end loop;
    end if;

    -- element 11
    if type1 = 1 then
      put(io,"layer make Initial_11 ");
      new_line(io);
      put(io,"-color");
      new_line(io);
      put(io,"red");
      new_line(io);

      -- diagonal
      case twist is
        when -1 =>
          for i in 1 .. ni / ns loop
            putelem1(i * ns,0,(i - 1) * ns,nj);
          end loop;

        when 0 =>
          for i in 0 .. ni / ns - 1 loop
            putelem1(i * ns,0,i * ns,nj);
          end loop;

        when 1 =>
          for i in 1 .. ni / ns loop
            putelem1((i - 1) * ns,0,i * ns,nj);
          end loop;

        when others =>
          null;
      end case;
    end if;

    -- element 12
    if type1 = 2 then
      put(io,"layer make Initial_12 ");
      new_line(io);
      put(io,"-color");
      new_line(io);
      put(io,"cyan");
      new_line(io);

      -- diagonal
      case twist is
        when -1 =>
          for i in 1 .. ni / ns loop
            putelem1(i * ns,0,(i - 1) * ns,nj);
          end loop;

        when 0 =>
          for i in 0 .. ni / ns - 1 loop
            putelem1(i * ns,0,i * ns,nj);
          end loop;

        when 1 =>
          for i in 1 .. ni / ns loop
            putelem1((i - 1) * ns,0,i * ns,nj);
          end loop;

        when others =>
          null;
      end case;
    end if;

    put(io,"-color");
    new_line(io);
    put(io,"white");
    new_line(io);

    put(io,"layer set 0 ");
    new_line(io);

    put(io,"zoom ext ext");
    new_line(io);

    close(io);
  end autocad;

begin
  initia;
  input;
  autocad;
end cylinder3;
