with
  text_io,
  ada.integer_text_io,
  ada.long_float_text_io,
  ada.numerics,
  ada.numerics.long_elementary_functions;

procedure mobius is

  type vec3 is array (1 .. 3) of long_float;

  ni,nj,nn,ne             : integer;
  radius,height,thickness : long_float;

  procedure scan (io : in text_io.file_type) is

    ascii : character;

  begin
    loop
      text_io.get(io,ascii);
      text_io.skip_line(io);
      exit when ascii = ':';
    end loop;
  end scan;

  function node (i,j : in integer) return integer is

  begin
    return ni * j + (i mod ni) + 1;
  end node;

  procedure vertex (i,j : in integer;
                    x   : out vec3) is

    use
      ada.numerics,
      ada.numerics.long_elementary_functions;

    t,s : long_float;

  begin
    t := 2.0 * pi / long_float(ni) * long_float(i mod ni);
    s := (long_float(j) / long_float(nj) - 0.5) * height;

    x(1) := (radius + s * cos(t / 2.0)) * cos(t);
    x(2) := (radius + s * cos(t / 2.0)) * sin(t);
    x(3) := s * sin(t / 2.0);
  end vertex;

  procedure initia is

    use
      text_io,
      ada.integer_text_io,
      ada.long_float_text_io;

    io : file_type;

  begin
    open(io, in_file, "mobius.txt");

    -- The circumference is divided in ni parts.
    -- The height is divided in nj parts.
    -- The numbers ni >= 4 and nj >= 2 must be even.

    scan(io);
    get(io,radius);
    get(io,ni);
    get(io,height);
    get(io,nj);
    get(io,thickness);
    skip_line(io);

    close(io);

    -- nn, ne
    nn := ni * (nj + 1);
    ne := 2 * ni * nj;
  end initia;

  procedure input is

    use
      text_io,
      ada.integer_text_io,
      ada.long_float_text_io;

    maxls,maxminor,maxlist,maxmajor : integer;
    lstol,maxnlag,maxncon,mu        : long_float;
    e,glob,i,j                      : integer;
    x                               : vec3;
    io                              : file_type;

  begin
    -- solution parameters
    lstol := 1.0E-01;
    maxls := 20;
    maxminor := 30 * nn;
    maxlist := 3;
    maxnlag := 1.0E-03;
    mu := 0.0;
    maxncon := 0.0;
    maxmajor := 0;

    create(io, out_file, "input.txt");

    put(io,"        lstol     maxls  maxminor   maxlist                mu           maxnlag           maxncon  maxmajor");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,lstol,2,6,3);
    put(io,"     ");
    put(io,maxls,5);
    put(io,"     ");
    put(io,maxminor,5);
    put(io,"     ");
    put(io,maxlist,5);
    put(io,"     ");
    put(io,mu,2,6,3);
    put(io,"     ");
    put(io,maxnlag,2,6,3);
    put(io,"     ");
    put(io,maxncon,2,6,3);
    put(io,"     ");
    put(io,maxmajor,5);
    new_line(io);

    -- node coordinates
    new_line(io);
    new_line(io);
    put(io," node           coord x           coord y           coord z");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,nn,5);
    new_line(io);

    new_line(io);

    for j in 0 .. nj loop
      for i in 0 .. ni - 1 loop
        vertex(i,j,x);
        put(io,node(i,j),5);
        put(io,"     ");
        put(io,x(1),2,6,3);
        put(io,"     ");
        put(io,x(2),2,6,3);
        put(io,"     ");
        put(io,x(3),2,6,3);
        new_line(io);
      end loop;
    end loop;

    -- element 11
    new_line(io);
    new_line(io);
    put(io,"elem1      node      node              area");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,"    0");
    new_line(io);

    -- element 12
    new_line(io);
    new_line(io);
    put(io,"elem2      node      node");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,"    0");
    new_line(io);

    -- element 21
    new_line(io);
    new_line(io);
    put(io," elem      node      node      node         thickness      type");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,ne,5);
    new_line(io);

    -- -----------
    -- | \3 | 2/ |
    -- | 4\ | /1 |
    -- ---(i,j)---
    -- | 5/ | \8 |
    -- | /6 | 7\ |
    -- -----------

    e := 0;
    new_line(io);

    j := 1;
    while j < nj loop

      i := 1;
      while i < ni - 2 loop
        -- 1
        e := e + 1;
        put(io,e,5);
        put(io,"     ");
        put(io,node(i,j),5);
        put(io,"     ");
        put(io,node(i + 1,j),5);
        put(io,"     ");
        put(io,node(i + 1,j + 1),5);
        put(io,"     ");
        put(io,thickness,2,6,3);
        new_line(io);

        -- 2
        e := e + 1;
        put(io,e,5);
        put(io,"     ");
        put(io,node(i,j),5);
        put(io,"     ");
        put(io,node(i + 1,j + 1),5);
        put(io,"     ");
        put(io,node(i,j + 1),5);
        put(io,"     ");
        put(io,thickness,2,6,3);
        new_line(io);

        -- 3
        e := e + 1;
        put(io,e,5);
        put(io,"     ");
        put(io,node(i,j),5);
        put(io,"     ");
        put(io,node(i,j + 1),5);
        put(io,"     ");
        put(io,node(i - 1,j + 1),5);
        put(io,"     ");
        put(io,thickness,2,6,3);
        new_line(io);

        -- 4
        e := e + 1;
        put(io,e,5);
        put(io,"     ");
        put(io,node(i,j),5);
        put(io,"     ");
        put(io,node(i - 1,j + 1),5);
        put(io,"     ");
        put(io,node(i - 1,j),5);
        put(io,"     ");
        put(io,thickness,2,6,3);
        new_line(io);

        -- 5
        e := e + 1;
        put(io,e,5);
        put(io,"     ");
        put(io,node(i,j),5);
        put(io,"     ");
        put(io,node(i - 1,j),5);
        put(io,"     ");
        put(io,node(i - 1,j - 1),5);
        put(io,"     ");
        put(io,thickness,2,6,3);
        new_line(io);

        -- 6
        e := e + 1;
        put(io,e,5);
        put(io,"     ");
        put(io,node(i,j),5);
        put(io,"     ");
        put(io,node(i - 1,j - 1),5);
        put(io,"     ");
        put(io,node(i,j - 1),5);
        put(io,"     ");
        put(io,thickness,2,6,3);
        new_line(io);

        -- 7
        e := e + 1;
        put(io,e,5);
        put(io,"     ");
        put(io,node(i,j),5);
        put(io,"     ");
        put(io,node(i,j - 1),5);
        put(io,"     ");
        put(io,node(i + 1,j - 1),5);
        put(io,"     ");
        put(io,thickness,2,6,3);
        new_line(io);

        -- 8
        e := e + 1;
        put(io,e,5);
        put(io,"     ");
        put(io,node(i,j),5);
        put(io,"     ");
        put(io,node(i + 1,j - 1),5);
        put(io,"     ");
        put(io,node(i + 1,j),5);
        put(io,"     ");
        put(io,thickness,2,6,3);
        new_line(io);

        i := i + 2;
      end loop;

      -- 1
      e := e + 1;
      put(io,e,5);
      put(io,"     ");
      put(io,node(i,j),5);
      put(io,"     ");
      put(io,node(i + 1,nj - j),5);
      put(io,"     ");
      put(io,node(i + 1,nj - (j + 1)),5);
      put(io,"     ");
      put(io,thickness,2,6,3);
      new_line(io);

      -- 2
      e := e + 1;
      put(io,e,5);
      put(io,"     ");
      put(io,node(i,j),5);
      put(io,"     ");
      put(io,node(i + 1,nj - (j + 1)),5);
      put(io,"     ");
      put(io,node(i,j + 1),5);
      put(io,"     ");
      put(io,thickness,2,6,3);
      new_line(io);

      -- 3
      e := e + 1;
      put(io,e,5);
      put(io,"     ");
      put(io,node(i,j),5);
      put(io,"     ");
      put(io,node(i,j + 1),5);
      put(io,"     ");
      put(io,node(i - 1,j + 1),5);
      put(io,"     ");
      put(io,thickness,2,6,3);
      new_line(io);

      -- 4
      e := e + 1;
      put(io,e,5);
      put(io,"     ");
      put(io,node(i,j),5);
      put(io,"     ");
      put(io,node(i - 1,j + 1),5);
      put(io,"     ");
      put(io,node(i - 1,j),5);
      put(io,"     ");
      put(io,thickness,2,6,3);
      new_line(io);

      -- 5
      e := e + 1;
      put(io,e,5);
      put(io,"     ");
      put(io,node(i,j),5);
      put(io,"     ");
      put(io,node(i - 1,j),5);
      put(io,"     ");
      put(io,node(i - 1,j - 1),5);
      put(io,"     ");
      put(io,thickness,2,6,3);
      new_line(io);

      -- 6
      e := e + 1;
      put(io,e,5);
      put(io,"     ");
      put(io,node(i,j),5);
      put(io,"     ");
      put(io,node(i - 1,j - 1),5);
      put(io,"     ");
      put(io,node(i,j - 1),5);
      put(io,"     ");
      put(io,thickness,2,6,3);
      new_line(io);

      -- 7
      e := e + 1;
      put(io,e,5);
      put(io,"     ");
      put(io,node(i,j),5);
      put(io,"     ");
      put(io,node(i,j - 1),5);
      put(io,"     ");
      put(io,node(i + 1,nj - (j - 1)),5);
      put(io,"     ");
      put(io,thickness,2,6,3);
      new_line(io);

      -- 8
      e := e + 1;
      put(io,e,5);
      put(io,"     ");
      put(io,node(i,j),5);
      put(io,"     ");
      put(io,node(i + 1,nj - (j - 1)),5);
      put(io,"     ");
      put(io,node(i + 1,nj - j),5);
      put(io,"     ");
      put(io,thickness,2,6,3);
      new_line(io);

      j := j + 2;
    end loop;

    -- element 22
    new_line(io);
    new_line(io);
    put(io,"elem2      node      node      node");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,"    0");
    new_line(io);

    -- element 31
    new_line(io);
    new_line(io);
    put(io,"elem1      node      node      node      node");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,"    0");
    new_line(io);

    -- element 32
    new_line(io);
    new_line(io);
    put(io,"elem2      node      node      node      node");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,"    0");
    new_line(io);

    -- known displacements
    new_line(io);
    new_line(io);
    put(io," node      axis             displ");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,6 * ni,5);
    new_line(io);

    new_line(io);

    -- bottom
    for i in 0 .. ni - 1 loop
      glob := node(i,0);

      put(io,glob,5);
      put(io,"         1      0.000000E+00");
      new_line(io);

      put(io,glob,5);
      put(io,"         2      0.000000E+00");
      new_line(io);

      put(io,glob,5);
      put(io,"         3      0.000000E+00");
      new_line(io);
    end loop;

    -- top
    for i in 0 .. ni - 1 loop
      glob := node(i,nj);

      put(io,glob,5);
      put(io,"         1      0.000000E+00");
      new_line(io);

      put(io,glob,5);
      put(io,"         2      0.000000E+00");
      new_line(io);

      put(io,glob,5);
      put(io,"         3      0.000000E+00");
      new_line(io);
    end loop;
  end input;

  procedure autocad is

    use
      text_io,
      ada.long_float_text_io;

    i,j : integer;
    x   : vec3;
    io  : file_type;

  begin
    create(io, out_file, "mobius.scr");

    -- set running object snap mode to NONE
    put(io,"osmode 0");
    new_line(io);

    -- element 21
    put(io,"-color");
    new_line(io);
    put(io,"green");
    new_line(io);

    put(io,"layer make Initial_21 ");
    new_line(io);

    -- -----------
    -- | \3 | 2/ |
    -- | 4\ | /1 |
    -- ---(i,j)---
    -- | 5/ | \8 |
    -- | /6 | 7\ |
    -- -----------

    j := 1;
    while j < nj loop

      i := 1;
      while i < ni - 2 loop
        -- 1
        put(io,"3dface ");

        vertex(i,j,x);
        put(io,x(1),1,8,3);
        put(io,",");
        put(io,x(2),1,8,3);
        put(io,",");
        put(io,x(3),1,8,3);
        put(io," ");

        vertex(i + 1,j,x);
        put(io,x(1),1,8,3);
        put(io,",");
        put(io,x(2),1,8,3);
        put(io,",");
        put(io,x(3),1,8,3);
        put(io," ");

        vertex(i + 1,j + 1,x);
        put(io,x(1),1,8,3);
        put(io,",");
        put(io,x(2),1,8,3);
        put(io,",");
        put(io,x(3),1,8,3);
        put(io," ");

        put(io," ");
        new_line(io);

        -- 2
        put(io,"3dface ");

        vertex(i,j,x);
        put(io,x(1),1,8,3);
        put(io,",");
        put(io,x(2),1,8,3);
        put(io,",");
        put(io,x(3),1,8,3);
        put(io," ");

        vertex(i + 1,j + 1,x);
        put(io,x(1),1,8,3);
        put(io,",");
        put(io,x(2),1,8,3);
        put(io,",");
        put(io,x(3),1,8,3);
        put(io," ");

        vertex(i,j + 1,x);
        put(io,x(1),1,8,3);
        put(io,",");
        put(io,x(2),1,8,3);
        put(io,",");
        put(io,x(3),1,8,3);
        put(io," ");

        put(io," ");
        new_line(io);

        -- 3
        put(io,"3dface ");

        vertex(i,j,x);
        put(io,x(1),1,8,3);
        put(io,",");
        put(io,x(2),1,8,3);
        put(io,",");
        put(io,x(3),1,8,3);
        put(io," ");

        vertex(i,j + 1,x);
        put(io,x(1),1,8,3);
        put(io,",");
        put(io,x(2),1,8,3);
        put(io,",");
        put(io,x(3),1,8,3);
        put(io," ");

        vertex(i - 1,j + 1,x);
        put(io,x(1),1,8,3);
        put(io,",");
        put(io,x(2),1,8,3);
        put(io,",");
        put(io,x(3),1,8,3);
        put(io," ");

        put(io," ");
        new_line(io);

        -- 4
        put(io,"3dface ");

        vertex(i,j,x);
        put(io,x(1),1,8,3);
        put(io,",");
        put(io,x(2),1,8,3);
        put(io,",");
        put(io,x(3),1,8,3);
        put(io," ");

        vertex(i - 1,j + 1,x);
        put(io,x(1),1,8,3);
        put(io,",");
        put(io,x(2),1,8,3);
        put(io,",");
        put(io,x(3),1,8,3);
        put(io," ");

        vertex(i - 1,j,x);
        put(io,x(1),1,8,3);
        put(io,",");
        put(io,x(2),1,8,3);
        put(io,",");
        put(io,x(3),1,8,3);
        put(io," ");

        put(io," ");
        new_line(io);

        -- 5
        put(io,"3dface ");

        vertex(i,j,x);
        put(io,x(1),1,8,3);
        put(io,",");
        put(io,x(2),1,8,3);
        put(io,",");
        put(io,x(3),1,8,3);
        put(io," ");

        vertex(i - 1,j,x);
        put(io,x(1),1,8,3);
        put(io,",");
        put(io,x(2),1,8,3);
        put(io,",");
        put(io,x(3),1,8,3);
        put(io," ");

        vertex(i - 1,j - 1,x);
        put(io,x(1),1,8,3);
        put(io,",");
        put(io,x(2),1,8,3);
        put(io,",");
        put(io,x(3),1,8,3);
        put(io," ");

        put(io," ");
        new_line(io);

        -- 6
        put(io,"3dface ");

        vertex(i,j,x);
        put(io,x(1),1,8,3);
        put(io,",");
        put(io,x(2),1,8,3);
        put(io,",");
        put(io,x(3),1,8,3);
        put(io," ");

        vertex(i - 1,j - 1,x);
        put(io,x(1),1,8,3);
        put(io,",");
        put(io,x(2),1,8,3);
        put(io,",");
        put(io,x(3),1,8,3);
        put(io," ");

        vertex(i,j - 1,x);
        put(io,x(1),1,8,3);
        put(io,",");
        put(io,x(2),1,8,3);
        put(io,",");
        put(io,x(3),1,8,3);
        put(io," ");

        put(io," ");
        new_line(io);

        -- 7
        put(io,"3dface ");

        vertex(i,j,x);
        put(io,x(1),1,8,3);
        put(io,",");
        put(io,x(2),1,8,3);
        put(io,",");
        put(io,x(3),1,8,3);
        put(io," ");

        vertex(i,j - 1,x);
        put(io,x(1),1,8,3);
        put(io,",");
        put(io,x(2),1,8,3);
        put(io,",");
        put(io,x(3),1,8,3);
        put(io," ");

        vertex(i + 1,j - 1,x);
        put(io,x(1),1,8,3);
        put(io,",");
        put(io,x(2),1,8,3);
        put(io,",");
        put(io,x(3),1,8,3);
        put(io," ");

        put(io," ");
        new_line(io);

        -- 8
        put(io,"3dface ");

        vertex(i,j,x);
        put(io,x(1),1,8,3);
        put(io,",");
        put(io,x(2),1,8,3);
        put(io,",");
        put(io,x(3),1,8,3);
        put(io," ");

        vertex(i + 1,j - 1,x);
        put(io,x(1),1,8,3);
        put(io,",");
        put(io,x(2),1,8,3);
        put(io,",");
        put(io,x(3),1,8,3);
        put(io," ");

        vertex(i + 1,j,x);
        put(io,x(1),1,8,3);
        put(io,",");
        put(io,x(2),1,8,3);
        put(io,",");
        put(io,x(3),1,8,3);
        put(io," ");

        put(io," ");
        new_line(io);

        i := i + 2;
      end loop;

      -- 1
      put(io,"3dface ");

      vertex(i,j,x);
      put(io,x(1),1,8,3);
      put(io,",");
      put(io,x(2),1,8,3);
      put(io,",");
      put(io,x(3),1,8,3);
      put(io," ");

      vertex(i + 1,nj - j,x);
      put(io,x(1),1,8,3);
      put(io,",");
      put(io,x(2),1,8,3);
      put(io,",");
      put(io,x(3),1,8,3);
      put(io," ");

      vertex(i + 1,nj - (j + 1),x);
      put(io,x(1),1,8,3);
      put(io,",");
      put(io,x(2),1,8,3);
      put(io,",");
      put(io,x(3),1,8,3);
      put(io," ");

      put(io," ");
      new_line(io);

      -- 2
      put(io,"3dface ");

      vertex(i,j,x);
      put(io,x(1),1,8,3);
      put(io,",");
      put(io,x(2),1,8,3);
      put(io,",");
      put(io,x(3),1,8,3);
      put(io," ");

      vertex(i + 1,nj - (j + 1),x);
      put(io,x(1),1,8,3);
      put(io,",");
      put(io,x(2),1,8,3);
      put(io,",");
      put(io,x(3),1,8,3);
      put(io," ");

      vertex(i,j + 1,x);
      put(io,x(1),1,8,3);
      put(io,",");
      put(io,x(2),1,8,3);
      put(io,",");
      put(io,x(3),1,8,3);
      put(io," ");

      put(io," ");
      new_line(io);

      -- 3
      put(io,"3dface ");

      vertex(i,j,x);
      put(io,x(1),1,8,3);
      put(io,",");
      put(io,x(2),1,8,3);
      put(io,",");
      put(io,x(3),1,8,3);
      put(io," ");

      vertex(i,j + 1,x);
      put(io,x(1),1,8,3);
      put(io,",");
      put(io,x(2),1,8,3);
      put(io,",");
      put(io,x(3),1,8,3);
      put(io," ");

      vertex(i - 1,j + 1,x);
      put(io,x(1),1,8,3);
      put(io,",");
      put(io,x(2),1,8,3);
      put(io,",");
      put(io,x(3),1,8,3);
      put(io," ");

      put(io," ");
      new_line(io);

      -- 4
      put(io,"3dface ");

      vertex(i,j,x);
      put(io,x(1),1,8,3);
      put(io,",");
      put(io,x(2),1,8,3);
      put(io,",");
      put(io,x(3),1,8,3);
      put(io," ");

      vertex(i - 1,j + 1,x);
      put(io,x(1),1,8,3);
      put(io,",");
      put(io,x(2),1,8,3);
      put(io,",");
      put(io,x(3),1,8,3);
      put(io," ");

      vertex(i - 1,j,x);
      put(io,x(1),1,8,3);
      put(io,",");
      put(io,x(2),1,8,3);
      put(io,",");
      put(io,x(3),1,8,3);
      put(io," ");

      put(io," ");
      new_line(io);

      -- 5
      put(io,"3dface ");

      vertex(i,j,x);
      put(io,x(1),1,8,3);
      put(io,",");
      put(io,x(2),1,8,3);
      put(io,",");
      put(io,x(3),1,8,3);
      put(io," ");

      vertex(i - 1,j,x);
      put(io,x(1),1,8,3);
      put(io,",");
      put(io,x(2),1,8,3);
      put(io,",");
      put(io,x(3),1,8,3);
      put(io," ");

      vertex(i - 1,j - 1,x);
      put(io,x(1),1,8,3);
      put(io,",");
      put(io,x(2),1,8,3);
      put(io,",");
      put(io,x(3),1,8,3);
      put(io," ");

      put(io," ");
      new_line(io);

      -- 6
      put(io,"3dface ");

      vertex(i,j,x);
      put(io,x(1),1,8,3);
      put(io,",");
      put(io,x(2),1,8,3);
      put(io,",");
      put(io,x(3),1,8,3);
      put(io," ");

      vertex(i - 1,j - 1,x);
      put(io,x(1),1,8,3);
      put(io,",");
      put(io,x(2),1,8,3);
      put(io,",");
      put(io,x(3),1,8,3);
      put(io," ");

      vertex(i,j - 1,x);
      put(io,x(1),1,8,3);
      put(io,",");
      put(io,x(2),1,8,3);
      put(io,",");
      put(io,x(3),1,8,3);
      put(io," ");

      put(io," ");
      new_line(io);

      -- 7
      put(io,"3dface ");

      vertex(i,j,x);
      put(io,x(1),1,8,3);
      put(io,",");
      put(io,x(2),1,8,3);
      put(io,",");
      put(io,x(3),1,8,3);
      put(io," ");

      vertex(i,j - 1,x);
      put(io,x(1),1,8,3);
      put(io,",");
      put(io,x(2),1,8,3);
      put(io,",");
      put(io,x(3),1,8,3);
      put(io," ");

      vertex(i + 1,nj - (j - 1),x);
      put(io,x(1),1,8,3);
      put(io,",");
      put(io,x(2),1,8,3);
      put(io,",");
      put(io,x(3),1,8,3);
      put(io," ");

      put(io," ");
      new_line(io);

      -- 8
      put(io,"3dface ");

      vertex(i,j,x);
      put(io,x(1),1,8,3);
      put(io,",");
      put(io,x(2),1,8,3);
      put(io,",");
      put(io,x(3),1,8,3);
      put(io," ");

      vertex(i + 1,nj - (j - 1),x);
      put(io,x(1),1,8,3);
      put(io,",");
      put(io,x(2),1,8,3);
      put(io,",");
      put(io,x(3),1,8,3);
      put(io," ");

      vertex(i + 1,nj - j,x);
      put(io,x(1),1,8,3);
      put(io,",");
      put(io,x(2),1,8,3);
      put(io,",");
      put(io,x(3),1,8,3);
      put(io," ");

      put(io," ");
      new_line(io);

      j := j + 2;
    end loop;

    put(io,"-color");
    new_line(io);
    put(io,"white");
    new_line(io);

    put(io,"layer make Edge ");
    new_line(io);

    -- bottom
    for i in 0 .. ni - 2 loop
      put(io,"3dpoly ");

      vertex(i,0,x);
      put(io,x(1),1,8,3);
      put(io,",");
      put(io,x(2),1,8,3);
      put(io,",");
      put(io,x(3),1,8,3);
      put(io," ");

      vertex(i + 1,0,x);
      put(io,x(1),1,8,3);
      put(io,",");
      put(io,x(2),1,8,3);
      put(io,",");
      put(io,x(3),1,8,3);
      put(io," ");

      new_line(io);
    end loop;

    put(io,"3dpoly ");

    vertex(ni - 1,0,x);
    put(io,x(1),1,8,3);
    put(io,",");
    put(io,x(2),1,8,3);
    put(io,",");
    put(io,x(3),1,8,3);
    put(io," ");

    vertex(ni,nj,x);
    put(io,x(1),1,8,3);
    put(io,",");
    put(io,x(2),1,8,3);
    put(io,",");
    put(io,x(3),1,8,3);
    put(io," ");

    new_line(io);

    -- top
    for i in 0 .. ni - 2 loop
      put(io,"3dpoly ");

      vertex(i,nj,x);
      put(io,x(1),1,8,3);
      put(io,",");
      put(io,x(2),1,8,3);
      put(io,",");
      put(io,x(3),1,8,3);
      put(io," ");

      vertex(i + 1,nj,x);
      put(io,x(1),1,8,3);
      put(io,",");
      put(io,x(2),1,8,3);
      put(io,",");
      put(io,x(3),1,8,3);
      put(io," ");

      new_line(io);
    end loop;

    put(io,"3dpoly ");

    vertex(ni - 1,nj,x);
    put(io,x(1),1,8,3);
    put(io,",");
    put(io,x(2),1,8,3);
    put(io,",");
    put(io,x(3),1,8,3);
    put(io," ");

    vertex(ni,0,x);
    put(io,x(1),1,8,3);
    put(io,",");
    put(io,x(2),1,8,3);
    put(io,",");
    put(io,x(3),1,8,3);
    put(io," ");

    new_line(io);

    put(io,"layer set 0 ");
    new_line(io);

    put(io,"zoom ext ext");
    new_line(io);

    close(io);
  end autocad;

begin
  initia;

  input;

  autocad;
end mobius;
