with
  text_io,
  ada.integer_text_io,
  ada.long_float_text_io,
  ada.numerics,
  ada.numerics.long_elementary_functions;

procedure noose1 is

  type vec3 is array (1 .. 3) of long_float;

  type xe is array (integer range <>, integer range <>) of long_float;

  type xe_ptr is access xe;

  n,i1,i2,nn            : integer;
  type1,type2,place     : integer;
  a1,b1,a2,b2,height,dx : long_float;
  area1,area2,thickness : long_float;
  xe1,xe2               : xe_ptr;

  procedure scan (io : in text_io.file_type) is

    ascii : character;

  begin
    loop
      text_io.get(io,ascii);
      text_io.skip_line(io);
      exit when ascii = ':';
    end loop;
  end scan;

  function node (i,j : in integer) return integer is

  begin
    return (i1 + i - 1) * (i - i1) / 2 * n + j mod (n * i) + 1;
  end node;

  procedure vertex (i,j : in integer;
                    x   : out vec3) is

    k     : integer;
    alpha : long_float;
    x1,x2 : vec3;

  begin
    k := (j mod (n * i)) / i;

    alpha := long_float(i - i1) / long_float(i2 - i1);
    for axis in 1 .. 3 loop
      x1(axis) := xe1(k,axis) * (1.0 - alpha) + xe2(k,axis) * alpha;
      x2(axis) := xe1((k + 1) mod n,axis) * (1.0 - alpha) + xe2((k + 1) mod n,axis) * alpha;
    end loop;

    alpha := long_float((j mod (n * i)) - k * i) / long_float(i);
    for axis in 1 .. 3 loop
      x(axis) := x1(axis) * (1.0 - alpha) + x2(axis) * alpha;
    end loop;
  end vertex;

  procedure initia is

    use
      text_io,
      ada.integer_text_io,
      ada.long_float_text_io;

    io : file_type;

  begin
    open(io, in_file, "noose1.txt");

    -- The number n must be even.
    scan(io);
    get(io,n);
    get(io,i1);
    get(io,i2);
    skip_line(io);

    -- boundary 1
    scan(io);
    get(io,area1);
    get(io,a1);
    get(io,b1);
    get(io,type1);
    get(io,height);
    get(io,dx);
    get(io,place);
    skip_line(io);

    -- boundary 2
    scan(io);
    get(io,area2);
    get(io,a2);
    get(io,b2);
    get(io,type2);
    skip_line(io);

    scan(io);
    get(io,thickness);
    skip_line(io);

    close(io);
  end initia;

  procedure setup is

    use
      ada.numerics,
      ada.numerics.long_elementary_functions;

    t,ct,st,r,cf,tf : long_float;

  begin
    -- nn
    nn := (i1 + i2) * (i2 - i1 + 1) / 2 * n;

    -- polygon'st vertices
    xe1 := new xe (0 .. n - 1, 1 .. 3);
    xe2 := new xe (0 .. n - 1, 1 .. 3);

    -- boundary 1
    case place is
      when 0 =>
        for k in 0 .. n - 1 loop
          t := 2.0 * pi / long_float(n) * long_float(k);
          ct := cos(t);
          st := sin(t);

          r := 1.0 / sqrt((ct / a1) ** 2 + (st / b1) ** 2);
          xe1(k,1) := r * ct + a1 - dx;
          xe1(k,2) := r * st;
          xe1(k,3) := 0.0;
        end loop;
      when 1 =>
        for k in 0 .. n - 1 loop
          t := 2.0 * pi / long_float(n) * long_float(k);
          ct := cos(t);
          st := sin(t);

          r := 1.0 / sqrt((ct / a1) ** 2 + (st / b1) ** 2);
          xe1(k,1) := r * ct + a1 - dx;
          xe1(k,2) := r * st;
          xe1(k,3) := height;
        end loop;
      when 2 =>
        tf := height / (a2 + dx);
        cf := 1.0 / sqrt(1.0 + tf ** 2);

        for k in 0 .. n - 1 loop
          t := 2.0 * pi / long_float(n) * long_float(k);
          ct := cos(t);
          st := sin(t);

          r := 1.0 / sqrt((ct / a1) ** 2 + (st / b1) ** 2);
          xe1(k,1) := (r * ct + a1) * cf - dx;
          xe1(k,2) := r * st;
          xe1(k,3) := (a2 - (r * ct + a1) * cf + dx) * tf;
        end loop;
      when others => null;
    end case;

    -- boundary 2
    for k in 0 .. n - 1 loop
      t := 2.0 * pi / long_float(n) * long_float(k);
      ct := cos(t);
      st := sin(t);

      r := 1.0 / sqrt((ct / a2) ** 2 + (st / b2) ** 2);
      xe2(k,1) := r * ct;
      xe2(k,2) := r * st;
      xe2(k,3) := 0.0;
    end loop;
  end setup;

  procedure input is

    use
      text_io,
      ada.integer_text_io,
      ada.long_float_text_io;

    maxls,maxminor,maxlist,maxmajor : integer;
    lstol,maxnlag,maxncon,mu        : long_float;
    ne,e,nk,glob,k                  : integer;
    x                               : vec3;
    io                              : file_type;

  begin
    -- solution parameters
    lstol := 1.0E-01;
    maxls := 20;
    maxminor := 30 * nn;
    maxlist := 3;
    maxnlag := 1.0E-03;
    if (type1 = 2) or (type2 = 2) then
      mu := 1.0E-03;
      maxncon := 1.0E-06;
      maxmajor := 0;
      if type1 = 2 then
        maxmajor := maxmajor + n * i1;
      end if;
      if type2 = 2 then
        maxmajor := maxmajor + n * i2;
      end if;
      maxmajor := 10 * maxmajor;
    else
      mu := 0.0;
      maxncon := 0.0;
      maxmajor := 0;
    end if;

    create(io, out_file, "input.txt");

    put(io,"        lstol     maxls  maxminor   maxlist                mu           maxnlag           maxncon  maxmajor");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,lstol,2,6,3);
    put(io,"     ");
    put(io,maxls,5);
    put(io,"     ");
    put(io,maxminor,5);
    put(io,"     ");
    put(io,maxlist,5);
    put(io,"     ");
    put(io,mu,2,6,3);
    put(io,"     ");
    put(io,maxnlag,2,6,3);
    put(io,"     ");
    put(io,maxncon,2,6,3);
    put(io,"     ");
    put(io,maxmajor,5);
    new_line(io);

    -- node coordinates
    new_line(io);
    new_line(io);
    put(io," node           coord x           coord y           coord z");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,nn,5);
    new_line(io);

    new_line(io);
    for i in i1 .. i2 loop
      for j in 0 .. i * n - 1 loop
        vertex(i,j,x);
        put(io,node(i,j),5);
        put(io,"     ");
        put(io,x(1),2,6,3);
        put(io,"     ");
        put(io,x(2),2,6,3);
        put(io,"     ");
        put(io,x(3),2,6,3);
        new_line(io);
      end loop;
    end loop;

    -- element 11
    ne := 0;
    if type1 = 1 then
      ne := ne + n * i1;
    end if;
    if type2 = 1 then
      ne := ne + n * i2;
    end if;

    new_line(io);
    new_line(io);
    put(io,"elem1      node      node              area");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,ne,5);
    new_line(io);

    if (type1 = 1) or (type2 = 1) then
      e := 0;
      new_line(io);

      -- boundary 1
      if type1 = 1 then
        for j in 0 .. n * i1 - 1 loop
          e := e + 1;
          put(io,e,5);
          put(io,"     ");
          put(io,node(i1,j),5);
          put(io,"     ");
          put(io,node(i1,j + 1),5);
          put(io,"     ");
          put(io,area1,2,6,3);
          new_line(io);
        end loop;
      end if;

      -- boundary 2
      if type2 = 1 then
        for j in 0 .. n * i2 - 1 loop
          e := e + 1;
          put(io,e,5);
          put(io,"     ");
          put(io,node(i2,j),5);
          put(io,"     ");
          put(io,node(i2,j + 1),5);
          put(io,"     ");
          put(io,area2,2,6,3);
          new_line(io);
        end loop;
      end if;
    end if;

    -- element 12
    ne := 0;
    if type1 = 2 then
      ne := ne + n * i1;
    end if;
    if type2 = 2 then
      ne := ne + n * i2;
    end if;

    new_line(io);
    new_line(io);
    put(io,"elem2      node      node");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,ne,5);
    new_line(io);

    if (type1 = 2) or (type2 = 2) then
      e := 0;
      new_line(io);

      -- boundary 1
      if type1 = 2 then
        for j in 0 .. n * i1 - 1 loop
          e := e + 1;
          put(io,e,5);
          put(io,"     ");
          put(io,node(i1,j),5);
          put(io,"     ");
          put(io,node(i1,j + 1),5);
          new_line(io);
        end loop;
      end if;

      -- boundary 2
      if type2 = 2 then
        for j in 0 .. n * i2 - 1 loop
          e := e + 1;
          put(io,e,5);
          put(io,"     ");
          put(io,node(i2,j),5);
          put(io,"     ");
          put(io,node(i2,j + 1),5);
          new_line(io);
        end loop;
      end if;
    end if;

    -- element 21
    ne := (i2 + i1) * (i2 - i1) * n;

    new_line(io);
    new_line(io);
    put(io,"elem1      node      node      node         thickness");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,ne,5);
    new_line(io);

    e := 0;
    new_line(io);

    for i in i1 + 1 .. i2 loop
      for j in 0 .. i * n - 1 loop
        k := j / i;
        e := e + 1;
        put(io,e,5);
        put(io,"     ");
        put(io,node(i - 1,j - k),5);
        put(io,"     ");
        put(io,node(i,j),5);
        put(io,"     ");
        put(io,node(i,j + 1),5);
        put(io,"     ");
        put(io,thickness,2,6,3);
        new_line(io);
      end loop;
    end loop;

    for i in i1 .. i2 - 1 loop
      for j in 0 .. i * n - 1 loop
        k := j / i;
        e := e + 1;
        put(io,e,5);
        put(io,"     ");
        put(io,node(i + 1,j + 1 + k),5);
        put(io,"     ");
        put(io,node(i,j + 1),5);
        put(io,"     ");
        put(io,node(i,j),5);
        put(io,"     ");
        put(io,thickness,2,6,3);
        new_line(io);
      end loop;
    end loop;

    -- element 22
    new_line(io);
    new_line(io);
    put(io,"elem2      node      node      node");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,"    0");
    new_line(io);

    -- element 31
    new_line(io);
    new_line(io);
    put(io,"elem1      node      node      node      node");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,"    0");
    new_line(io);

    -- element 32
    new_line(io);
    new_line(io);
    put(io,"elem2      node      node      node      node");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,"    0");
    new_line(io);

    -- known displacements
    nk := 0;
    if type1 > 0 then
      nk := nk + 1;
    else
      nk := nk + n * i1;
    end if;
    if type2 > 0 then
      nk := nk + n;
    else
      nk := nk + n * i2;
    end if;
    nk := 3 * nk;

    new_line(io);
    new_line(io);
    put(io," node      axis             displ");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,nk,5);
    new_line(io);

    new_line(io);

    -- boundary 1
    if type1 > 0 then
      glob := node(i1,n * i1 / 2);
      put(io,glob,5);
      put(io,"         1      0.000000E+00");
      new_line(io);
      put(io,glob,5);
      put(io,"         2      0.000000E+00");
      new_line(io);
      put(io,glob,5);
      if place = 0 then
        put(io,"         3     ");
        put(io,height,2,6,3);
      else
        put(io,"         3      0.000000E+00");
      end if;
      new_line(io);
    else
      for j in 0 .. n * i1 - 1 loop
        glob := node(i1,j);
        put(io,glob,5);
        put(io,"         1      0.000000E+00");
        new_line(io);
        put(io,glob,5);
        put(io,"         2      0.000000E+00");
        new_line(io);
        put(io,glob,5);
        put(io,"         3      0.000000E+00");
        new_line(io);
      end loop;
    end if;

    -- boundary 2
    if type2 > 0 then
      for k in 0 .. n - 1 loop
        glob := node(i2,k * i2);
        put(io,glob,5);
        put(io,"         1      0.000000E+00");
        new_line(io);
        put(io,glob,5);
        put(io,"         2      0.000000E+00");
        new_line(io);
        put(io,glob,5);
        put(io,"         3      0.000000E+00");
        new_line(io);
      end loop;
    else
      for j in 0 .. n * i2 - 1 loop
        glob := node(i2,j);
        put(io,glob,5);
        put(io,"         1      0.000000E+00");
        new_line(io);
        put(io,glob,5);
        put(io,"         2      0.000000E+00");
        new_line(io);
        put(io,glob,5);
        put(io,"         3      0.000000E+00");
        new_line(io);
      end loop;
    end if;

    close(io);
  end input;

  procedure autocad is

    use
      text_io,
      ada.long_float_text_io,
      ada.numerics.long_elementary_functions;

    k              : integer;
    factor,sc1,sc2 : long_float;
    x,xmin,xmax    : vec3;
    io             : file_type;

  procedure xsupport (x,y,z : in long_float) is

  begin
    put(io,"3dface ");

    put(io,x,1,6,3);
    put(io,",");
    put(io,y,1,6,3);
    put(io,",");
    put(io,z,1,6,3);
    put(io," ");

    put(io,"@");
    put(io,sc2,1,6,3);
    put(io,",");
    put(io,-sc1,1,6,3);
    put(io,",");
    put(io,"0.00000000E+00");
    put(io," ");

    put(io,"@");
    put(io,"0.00000000E+00");
    put(io,",");
    put(io,(sc1 + sc1),1,6,3);
    put(io,",");
    put(io,"0.00000000E+00");
    put(io," ");

    put(io," ");
    new_line(io);
  end xsupport;

  procedure ysupport (x,y,z : in long_float) is

  begin
    put(io,"3dface ");

    put(io,x,1,6,3);
    put(io,",");
    put(io,y,1,6,3);
    put(io,",");
    put(io,z,1,6,3);
    put(io," ");

    put(io,"@");
    put(io,"0.00000000E+00");
    put(io,",");
    put(io,sc2,1,6,3);
    put(io,",");
    put(io,-sc1,1,6,3);
    put(io," ");

    put(io,"@");
    put(io,"0.00000000E+00");
    put(io,",");
    put(io,"0.00000000E+00");
    put(io,",");
    put(io,(sc1 + sc1),1,6,3);
    put(io," ");

    put(io," ");
    new_line(io);
  end ysupport;

  procedure zsupport (x,y,z : in long_float) is

  begin
    put(io,"3dface ");

    put(io,x,1,6,3);
    put(io,",");
    put(io,y,1,6,3);
    put(io,",");
    put(io,z,1,6,3);
    put(io," ");

    put(io,"@");
    put(io,-sc1,1,6,3);
    put(io,",");
    put(io,"0.00000000E+00");
    put(io,",");
    put(io,sc2,1,6,3);
    put(io," ");

    put(io,"@");
    put(io,(sc1 + sc1),1,6,3);
    put(io,",");
    put(io,"0.00000000E+00");
    put(io,",");
    put(io,"0.00000000E+00");
    put(io," ");

    put(io," ");
    new_line(io);
  end zsupport;

  begin
    -- bounding box
    vertex(i1,0,x);
    for axis in 1 .. 3 loop
      xmin(axis) := x(axis);
      xmax(axis) := x(axis);
    end loop;

    for i in i1 .. i2 loop
      for j in 0 .. i * n - 1 loop
        vertex(i,j,x);
        for axis in 1 .. 3 loop
          if x(axis) < xmin(axis) then
            xmin(axis) := x(axis);
          end if;
          if x(axis) > xmax(axis) then
            xmax(axis) := x(axis);
          end if;
        end loop;
      end loop;
    end loop;

    -- scale factor
    factor := sqrt((xmax(1) - xmin(1)) ** 2 + (xmax(2) - xmin(2)) ** 2 + (xmax(3) - xmin(3)) ** 2);
    factor := factor / sqrt(2.0) / 50.0;
    sc1 := 0.5 * factor;
    sc2 := sqrt(3.0) / 2.0 * factor;

    create(io, out_file, "noose1.scr");

    -- set running object snap mode to none
    put(io,"osmode 0");
    new_line(io);

    -- element 21
    put(io,"-color");
    new_line(io);
    put(io,"green");
    new_line(io);

    put(io,"layer make Initial_21 ");
    new_line(io);

    for i in i1 + 1 .. i2 loop
      for j in 0 .. i * n - 1 loop
        k := j / i;

        put(io,"3dface ");

        vertex(i - 1,j - k,x);
        put(io,x(1),1,6,3);
        put(io,",");
        put(io,x(2),1,6,3);
        put(io,",");
        put(io,x(3),1,6,3);
        put(io," ");

        vertex(i,j,x);
        put(io,x(1),1,6,3);
        put(io,",");
        put(io,x(2),1,6,3);
        put(io,",");
        put(io,x(3),1,6,3);
        put(io," ");

        vertex(i,j + 1,x);
        put(io,x(1),1,6,3);
        put(io,",");
        put(io,x(2),1,6,3);
        put(io,",");
        put(io,x(3),1,6,3);
        put(io," ");

        put(io," ");
        new_line(io);
      end loop;
    end loop;

    for i in i1 .. i2 - 1 loop
      for j in 0 .. i * n - 1 loop
        k := j / i;

        put(io,"3dface ");

        vertex(i + 1,j + 1 + k,x);
        put(io,x(1),1,6,3);
        put(io,",");
        put(io,x(2),1,6,3);
        put(io,",");
        put(io,x(3),1,6,3);
        put(io," ");

        vertex(i,j + 1,x);
        put(io,x(1),1,6,3);
        put(io,",");
        put(io,x(2),1,6,3);
        put(io,",");
        put(io,x(3),1,6,3);
        put(io," ");

        vertex(i,j,x);
        put(io,x(1),1,6,3);
        put(io,",");
        put(io,x(2),1,6,3);
        put(io,",");
        put(io,x(3),1,6,3);
        put(io," ");

        put(io," ");
        new_line(io);
      end loop;
    end loop;

    -- element 11
    if (type1 = 1) or (type2 = 1) then
      put(io,"layer make Initial_11 ");
      new_line(io);
      put(io,"-color");
      new_line(io);
      put(io,"red");
      new_line(io);

      -- boundary 1
      if type1 = 1 then
        for j in 0 .. n * i1 - 1 loop
          put(io,"3dpoly ");
          vertex(i1,j,x);
          put(io,x(1),1,6,3);
          put(io,",");
          put(io,x(2),1,6,3);
          put(io,",");
          put(io,x(3),1,6,3);
          put(io," ");
          vertex(i1,j + 1,x);
          put(io,x(1),1,6,3);
          put(io,",");
          put(io,x(2),1,6,3);
          put(io,",");
          put(io,x(3),1,6,3);
          put(io," ");
          new_line(io);
        end loop;
      end if;

      -- boundary 2
      if type2 = 1 then
        for j in 0 .. n * i2 - 1 loop
          put(io,"3dpoly ");
          vertex(i2,j,x);
          put(io,x(1),1,6,3);
          put(io,",");
          put(io,x(2),1,6,3);
          put(io,",");
          put(io,x(3),1,6,3);
          put(io," ");
          vertex(i2,j + 1,x);
          put(io,x(1),1,6,3);
          put(io,",");
          put(io,x(2),1,6,3);
          put(io,",");
          put(io,x(3),1,6,3);
          put(io," ");
          new_line(io);
        end loop;
      end if;
    end if;

    -- element 12
    if (type1 = 2) or (type2 = 2) then
      put(io,"layer make Initial_12 ");
      new_line(io);
      put(io,"-color");
      new_line(io);
      put(io,"cyan");
      new_line(io);

      -- boundary 1
      if type1 = 2 then
        for j in 0 .. n * i1 - 1 loop
          put(io,"3dpoly ");
          vertex(i1,j,x);
          put(io,x(1),1,6,3);
          put(io,",");
          put(io,x(2),1,6,3);
          put(io,",");
          put(io,x(3),1,6,3);
          put(io," ");
          vertex(i1,j + 1,x);
          put(io,x(1),1,6,3);
          put(io,",");
          put(io,x(2),1,6,3);
          put(io,",");
          put(io,x(3),1,6,3);
          put(io," ");
          new_line(io);
        end loop;
      end if;

      -- boundary 2
      if type2 = 2 then
        for j in 0 .. n * i2 - 1 loop
          put(io,"3dpoly ");
          vertex(i2,j,x);
          put(io,x(1),1,6,3);
          put(io,",");
          put(io,x(2),1,6,3);
          put(io,",");
          put(io,x(3),1,6,3);
          put(io," ");
          vertex(i2,j + 1,x);
          put(io,x(1),1,6,3);
          put(io,",");
          put(io,x(2),1,6,3);
          put(io,",");
          put(io,x(3),1,6,3);
          put(io," ");
          new_line(io);
        end loop;
      end if;
    end if;

    -- support
    put(io,"-color");
    new_line(io);
    put(io,"white");
    new_line(io);

    put(io,"layer make Support ");
    new_line(io);

    -- boundary 1
    if type1 > 0 then
      vertex(i1,n * i1 / 2,x);
      if place = 0 then
        put(io,"3dpoly ");
        put(io,x(1),1,6,3);
        put(io,",");
        put(io,x(2),1,6,3);
        put(io,",");
        put(io,x(3),1,6,3);
        put(io," ");
        x(3) := x(3) + height;
        put(io,x(1),1,6,3);
        put(io,",");
        put(io,x(2),1,6,3);
        put(io,",");
        put(io,x(3),1,6,3);
        put(io," ");
        new_line(io);
      end if;
      xsupport(x(1),x(2),x(3));
      ysupport(x(1),x(2),x(3));
      zsupport(x(1),x(2),x(3));
    else
      for j in 0 .. n * i1 - 1 loop
        vertex(i1,j,x);
        xsupport(x(1),x(2),x(3));
        ysupport(x(1),x(2),x(3));
        zsupport(x(1),x(2),x(3));
      end loop;
    end if;

    -- boundary 2
    if type2 > 0 then
      for k in 0 .. n - 1 loop
        vertex(i2,k * i2,x);
        xsupport(x(1),x(2),x(3));
        ysupport(x(1),x(2),x(3));
        zsupport(x(1),x(2),x(3));
      end loop;
    else
      for j in 0 .. n * i2 - 1 loop
        vertex(i2,j,x);
        xsupport(x(1),x(2),x(3));
        ysupport(x(1),x(2),x(3));
        zsupport(x(1),x(2),x(3));
      end loop;
    end if;

    put(io,"layer set 0 ");
    new_line(io);

    put(io,"zoom ext ext");
    new_line(io);

    close(io);
  end autocad;

begin
  initia;
  setup;
  input;
  autocad;
end noose1;
