with
  text_io,
  ada.integer_text_io,
  ada.long_float_text_io,
  ada.numerics,
  ada.numerics.long_elementary_functions;

procedure sprism1 is

  type vec3 is array (1 .. 3) of long_float;

  type fmat1 is array (integer range <>) of long_float;

  type fmat1_ptr is access fmat1;

  np,nc       : integer;
  height,area : long_float;
  rc          : fmat1_ptr;

  procedure scan (io : in text_io.file_type) is

    ascii : character;

  begin
    loop
      text_io.get(io,ascii);
      text_io.skip_line(io);
      exit when ascii = ':';
    end loop;
  end scan;

  function spot (i,j : in integer) return integer is

  begin
    return nc * i + j mod nc + 1;
  end spot;

  procedure vertex (i,j : in integer;
                    x   : out vec3) is

    use
      ada.numerics,
      ada.numerics.long_elementary_functions;

    aj : long_float;

  begin
    aj := (pi + pi) / long_float(nc) * long_float(j);
    x(1) := rc(i) * cos(aj);
    x(2) := rc(i) * sin(aj);
    x(3) := height * long_float(i);
  end vertex;

  procedure initia is

    use
      text_io,
      ada.integer_text_io,
      ada.long_float_text_io;

    io : file_type;

  begin
    open(io, in_file, "sprism1.txt");

    -- The structure is composed of regular prisms stacked one on top of the other
    -- np : number of prisms
    -- nc : number of sides of the inscribed polygon
    scan(io);
    get(io,np);
    get(io,nc);
    get(io,height);
    get(io,area);
    skip_line(io);

    -- rc : radius of the polygon
    rc := new fmat1 (0 .. np);

    scan(io);
    for i in 0 .. np loop
      get(io,rc(i));
      skip_line(io);
    end loop;

    close(io);
  end initia;

  procedure input is

    use
      text_io,
      ada.integer_text_io,
      ada.long_float_text_io;

    maxls,maxminor,maxlist,maxmajor : integer;
    lstol,maxnlag,maxncon,mu        : long_float;
    e                               : integer;
    x                               : vec3;
    io                              : file_type;

  begin
    -- solution parameters
    lstol := 1.0E-01;
    maxls := 20;
    maxminor := 30 * (np + 1) * nc;
    maxlist := 3;
    mu := 1.0E-03;
    maxnlag := 1.0E-05;
    maxncon := 1.0E-06;
    maxmajor := 10 * (np + np + 1) * nc;

    create(io, out_file, "input.txt");

    put(io,"        lstol     maxls  maxminor   maxlist                mu           maxnlag           maxncon  maxmajor");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,lstol,2,6,3);
    put(io,"     ");
    put(io,maxls,5);
    put(io,"     ");
    put(io,maxminor,5);
    put(io,"     ");
    put(io,maxlist,5);
    put(io,"     ");
    put(io,mu,2,6,3);
    put(io,"     ");
    put(io,maxnlag,2,6,3);
    put(io,"     ");
    put(io,maxncon,2,6,3);
    put(io,"     ");
    put(io,maxmajor,5);
    new_line(io);

    -- node coordinates
    new_line(io);
    new_line(io);
    put(io," node           coord x           coord y           coord z");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,(np + 1) * nc,5);
    new_line(io);

    new_line(io);
    for i in 0 .. np loop
      for j in 0 .. nc - 1 loop
        vertex(i,j,x);
        put(io,spot(i,j),5);
        put(io,"     ");
        put(io,x(1),2,6,3);
        put(io,"     ");
        put(io,x(2),2,6,3);
        put(io,"     ");
        put(io,x(3),2,6,3);
        new_line(io);
      end loop;
    end loop;

    -- element 11
    new_line(io);
    new_line(io);
    put(io,"elem1      node      node              area");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,np * nc,5);
    new_line(io);

    new_line(io);
    e := 0;

    -- diagonal
      for i in 0 .. np - 1 loop
        if i mod 2 = 0 then
          for j in 0 .. nc - 1 loop
            e := e + 1;

            put(io,e,5);
            put(io,"     ");
            put(io,spot(i,j),5);
            put(io,"     ");
            put(io,spot(i + 1,j + 1),5);
            put(io,"     ");
            put(io,area,2,6,3);
            new_line(io);
          end loop;
        else
          for j in 0 .. nc - 1 loop
            e := e + 1;

            put(io,e,5);
            put(io,"     ");
            put(io,spot(i,j + 1),5);
            put(io,"     ");
            put(io,spot(i + 1,j),5);
            put(io,"     ");
            put(io,area,2,6,3);
            new_line(io);
          end loop;
        end if;
      end loop;

    -- element 12
    new_line(io);
    new_line(io);
    put(io,"elem2      node      node");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,(np + np + 1) * nc,5);
    new_line(io);

    new_line(io);
    e := 0;

    -- polygon
    for i in 0 .. np loop
      for j in 0 .. nc - 1 loop
        e := e + 1;

        put(io,e,5);
        put(io,"     ");
        put(io,spot(i,j),5);
        put(io,"     ");
        put(io,spot(i,j + 1),5);
        new_line(io);
      end loop;
    end loop;

    -- vertical
    for i in 0 .. np - 1 loop
      for j in 0 .. nc - 1 loop
        e := e + 1;

        put(io,e,5);
        put(io,"     ");
        put(io,spot(i,j),5);
        put(io,"     ");
        put(io,spot(i + 1,j),5);
        new_line(io);
      end loop;
    end loop;

    -- element 21
    new_line(io);
    new_line(io);
    put(io,"elem1      node      node      node         thickness");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,"    0");
    new_line(io);

    -- element 22
    new_line(io);
    new_line(io);
    put(io,"elem2      node      node      node");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,"    0");
    new_line(io);

    -- element 31
    new_line(io);
    new_line(io);
    put(io,"elem1      node      node      node      node");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,"    0");
    new_line(io);

    -- element 32
    new_line(io);
    new_line(io);
    put(io,"elem2      node      node      node      node");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,"    0");
    new_line(io);

    -- known displacements
    new_line(io);
    new_line(io);
    put(io," node      axis             displ");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,"    0");
    new_line(io);

    close(io);
  end input;

  procedure autocad is

    use
      text_io,
      ada.integer_text_io,
      ada.long_float_text_io;

    x  : vec3;
    io : file_type;

  begin
    create(io, out_file, "sprism1.scr");

    -- set running object snap mode to NONE
    put(io,"osmode 0");
    new_line(io);

    -- element 11
    put(io,"-color");
    new_line(io);
    put(io,"red");
    new_line(io);

    put(io,"layer make Initial_11 ");
    new_line(io);

    -- diagonal
      for i in 0 .. np - 1 loop
        if i mod 2 = 0 then
          for j in 0 .. nc - 1 loop
            put(io,"3dpoly ");

            vertex(i,j,x);
            put(io,x(1),1,8,3);
            put(io,",");
            put(io,x(2),1,8,3);
            put(io,",");
            put(io,x(3),1,8,3);
            put(io," ");

            vertex(i + 1,j + 1,x);
            put(io,x(1),1,8,3);
            put(io,",");
            put(io,x(2),1,8,3);
            put(io,",");
            put(io,x(3),1,8,3);
            put(io," ");

            new_line(io);
          end loop;
        else
          for j in 0 .. nc - 1 loop
            put(io,"3dpoly ");

            vertex(i,j + 1,x);
            put(io,x(1),1,8,3);
            put(io,",");
            put(io,x(2),1,8,3);
            put(io,",");
            put(io,x(3),1,8,3);
            put(io," ");

            vertex(i + 1,j,x);
            put(io,x(1),1,8,3);
            put(io,",");
            put(io,x(2),1,8,3);
            put(io,",");
            put(io,x(3),1,8,3);
            put(io," ");

            new_line(io);
          end loop;
        end if;
      end loop;

    -- element 12
    put(io,"-color");
    new_line(io);
    put(io,"cyan");
    new_line(io);

    put(io,"layer make Initial_12 ");
    new_line(io);

    -- polygon
    for i in 0 .. np loop
      for j in 0 .. nc - 1 loop
        put(io,"3dpoly ");

        vertex(i,j,x);
        put(io,x(1),1,8,3);
        put(io,",");
        put(io,x(2),1,8,3);
        put(io,",");
        put(io,x(3),1,8,3);
        put(io," ");

        vertex(i,j + 1,x);
        put(io,x(1),1,8,3);
        put(io,",");
        put(io,x(2),1,8,3);
        put(io,",");
        put(io,x(3),1,8,3);
        put(io," ");

        new_line(io);
      end loop;
    end loop;

    -- vertical
    for i in 0 .. np - 1 loop
      for j in 0 .. nc - 1 loop
        put(io,"3dpoly ");

        vertex(i,j,x);
        put(io,x(1),1,8,3);
        put(io,",");
        put(io,x(2),1,8,3);
        put(io,",");
        put(io,x(3),1,8,3);
        put(io," ");

        vertex(i + 1,j,x);
        put(io,x(1),1,8,3);
        put(io,",");
        put(io,x(2),1,8,3);
        put(io,",");
        put(io,x(3),1,8,3);
        put(io," ");

        new_line(io);
      end loop;
    end loop;

    put(io,"layer set 0 ");
    new_line(io);

    put(io,"zoom ext ext");
    new_line(io);

    close(io);
  end autocad;

begin
  initia;
  input;
  autocad;
end sprism1;
