with
  text_io,
  ada.integer_text_io,
  ada.long_float_text_io,
  ada.numerics.long_elementary_functions;

procedure stella is

  type ivec2 is array (1 .. 2) of integer;

  type elem11_rec is
    record
      glob : ivec2;
      area : long_float := 1.0;
    end record;

  type elem12_rec is
    record
      glob : ivec2;
    end record;

  xo,yo,zo : array (1..12) of long_float;
  elem11   : array (1..6) of elem11_rec;
  elem12   : array (1..18) of elem12_rec;

  procedure scan (io : in text_io.file_type) is

    ascii : character;

  begin
    loop
      text_io.get(io,ascii);
      text_io.skip_line(io);
      exit when ascii = ':';
    end loop;
  end scan;

  procedure initia is

    use
      text_io,
      ada.integer_text_io,
      ada.long_float_text_io,
      ada.numerics.long_elementary_functions;

    e     : integer;
    s,r,h : long_float;
    io    : file_type;

  begin
    open(io, in_file, "stella.txt");

    -- s
    scan(io);
    get(io,s);
    skip_line(io);

    -- area
    scan(io);

    skip_line(io);
    for k in 1 .. 6 loop
      get(io,e);
      get(io,elem11(e).area);
      skip_line(io);
    end loop;

    close(io);

    -- nodes
    r := s / sqrt(3.0);
    h := s / sqrt(6.0);

    xo(1) := -s / 2.0;
    yo(1) := -r / 2.0;
    zo(1) := h;

    xo(2) := s / 2.0;
    yo(2) := -r / 2.0;
    zo(2) := h;

    xo(3) := 0.0;
    yo(3) := r;
    zo(3) := h;

    xo(4) := 0.0;
    yo(4) := -2.0 * r;
    zo(4) := h;

    xo(5) := s;
    yo(5) := r;
    zo(5) := h;

    xo(6) := -s;
    yo(6) := r;
    zo(6) := h;

    xo(7) := s;
    yo(7) := -r;
    zo(7) := -h;

    xo(8) := -s;
    yo(8) := -r;
    zo(8) := -h;

    xo(9) := 0.0;
    yo(9) := 2.0 * r;
    zo(9) := -h;

    xo(10) := 0.0;
    yo(10) := -r;
    zo(10) := -h;

    xo(11) := -s / 2.0;
    yo(11) := r / 2.0;
    zo(11) := -h;

    xo(12) := s / 2.0;
    yo(12) := r / 2.0;
    zo(12) := -h;

    -- element 11
    elem11(1).glob(1) := 4;
    elem11(1).glob(2) := 11;

    elem11(2).glob(1) := 5;
    elem11(2).glob(2) := 10;

    elem11(3).glob(1) := 6;
    elem11(3).glob(2) := 12;

    elem11(4).glob(1) := 7;
    elem11(4).glob(2) := 1;

    elem11(5).glob(1) := 8;
    elem11(5).glob(2) := 3;

    elem11(6).glob(1) := 9;
    elem11(6).glob(2) := 2;

    -- element 12
    elem12(1).glob(1) := 4;
    elem12(1).glob(2) := 1;

    elem12(2).glob(1) := 4;
    elem12(2).glob(2) := 10;

    elem12(3).glob(1) := 5;
    elem12(3).glob(2) := 2;

    elem12(4).glob(1) := 5;
    elem12(4).glob(2) := 12;

    elem12(5).glob(1) := 6;
    elem12(5).glob(2) := 3;

    elem12(6).glob(1) := 6;
    elem12(6).glob(2) := 11;

    elem12(7).glob(1) := 7;
    elem12(7).glob(2) := 10;

    elem12(8).glob(1) := 7;
    elem12(8).glob(2) := 2;

    elem12(9).glob(1) := 8;
    elem12(9).glob(2) := 11;

    elem12(10).glob(1) := 8;
    elem12(10).glob(2) := 1;

    elem12(11).glob(1) := 9;
    elem12(11).glob(2) := 12;

    elem12(12).glob(1) := 9;
    elem12(12).glob(2) := 3;

    elem12(13).glob(1) := 1;
    elem12(13).glob(2) := 2;

    elem12(14).glob(1) := 2;
    elem12(14).glob(2) := 3;

    elem12(15).glob(1) := 3;
    elem12(15).glob(2) := 1;

    elem12(16).glob(1) := 10;
    elem12(16).glob(2) := 11;

    elem12(17).glob(1) := 11;
    elem12(17).glob(2) := 12;

    elem12(18).glob(1) := 12;
    elem12(18).glob(2) := 10;
  end initia;

  procedure input is

    use
      text_io,
      ada.integer_text_io,
      ada.long_float_text_io;

    maxls,maxminor,maxlist,maxmajor : integer;
    lstol,maxnlag,maxncon,mu        : long_float;
    io                              : file_type;

  begin
    -- solution parameters
    lstol := 1.0E-01;
    maxls := 20;
    maxminor := 360;
    maxlist := 3;
    mu := 1.0E-03;
    maxnlag := 1.0E-03;
    maxncon := 1.0E-04;
    maxmajor := 180;

    create(io, out_file, "input.txt");

    put(io,"        lstol     maxls  maxminor   maxlist                mu           maxnlag           maxncon  maxmajor");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,lstol,2,6,3);
    put(io,"     ");
    put(io,maxls,5);
    put(io,"     ");
    put(io,maxminor,5);
    put(io,"     ");
    put(io,maxlist,5);
    put(io,"     ");
    put(io,mu,2,6,3);
    put(io,"     ");
    put(io,maxnlag,2,6,3);
    put(io,"     ");
    put(io,maxncon,2,6,3);
    put(io,"     ");
    put(io,maxmajor,5);
    new_line(io);

    -- node coordinates
    new_line(io);
    new_line(io);
    put(io," node           coord x           coord y           coord z");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,"   12");
    new_line(io);

    new_line(io);
    for glob in 1 .. 12 loop
      put(io,glob,5);
      put(io,"     ");
      put(io,xo(glob),2,6,3);
      put(io,"     ");
      put(io,yo(glob),2,6,3);
      put(io,"     ");
      put(io,zo(glob),2,6,3);
      new_line(io);
    end loop;

    -- element 11
    new_line(io);
    new_line(io);
    put(io,"elem1      node      node              area");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,"    6");
    new_line(io);

    new_line(io);
    for e in elem11'range loop
      put(io,e,5);
      put(io,"     ");
      put(io,elem11(e).glob(1),5);
      put(io,"     ");
      put(io,elem11(e).glob(2),5);
      put(io,"     ");
      put(io,elem11(e).area,2,6,3);
      new_line(io);
    end loop;

    -- element 12
    new_line(io);
    new_line(io);
    put(io,"elem2      node      node");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,"   18");
    new_line(io);

    new_line(io);
    for e in elem12'range loop
      put(io,e,5);
      put(io,"     ");
      put(io,elem12(e).glob(1),5);
      put(io,"     ");
      put(io,elem12(e).glob(2),5);
      new_line(io);
    end loop;

    -- element 21
    new_line(io);
    new_line(io);
    put(io,"elem1      node      node      node         thickness");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,"    0");
    new_line(io);

    -- element 22
    new_line(io);
    new_line(io);
    put(io,"elem2      node      node      node");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,"    0");
    new_line(io);

    -- element 31
    new_line(io);
    new_line(io);
    put(io,"elem1      node      node      node      node");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,"    0");
    new_line(io);

    -- element 32
    new_line(io);
    new_line(io);
    put(io,"elem2      node      node      node      node");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,"    0");
    new_line(io);

    -- known displacements
    new_line(io);
    new_line(io);
    put(io," node      axis             displ");
    new_line(io);
    put(io,":");
    new_line(io);
    put(io,"    6");
    new_line(io);

    new_line(io);
    put(io,"    1         1      0.000000E+00");
    new_line(io);
    put(io,"    1         2      0.000000E+00");
    new_line(io);
    put(io,"    1         3      0.000000E+00");
    new_line(io);
    put(io,"    2         2      0.000000E+00");
    new_line(io);
    put(io,"    2         3      0.000000E+00");
    new_line(io);
    put(io,"    3         3      0.000000E+00");
    new_line(io);

    close(io);
  end input;

  procedure autocad is

    use
      text_io,
      ada.integer_text_io,
      ada.long_float_text_io;

    glob : integer;
    io   : file_type;

  begin
    create(io, out_file, "stella.scr");

    -- set running object snap mode to NONE
    put(io,"osmode 0");
    new_line(io);

    -- elements 1
    put(io,"-color");
    new_line(io);
    put(io,"red");
    new_line(io);

    put(io,"layer make Initial_11 ");
    new_line(io);

    for e in elem11'range loop
      put(io,"3dpoly ");
      for loc in 1 .. 2 loop
        glob := elem11(e).glob(loc);
        put(io,xo(glob),1,8,3);
        put(io,",");
        put(io,yo(glob),1,8,3);
        put(io,",");
        put(io,zo(glob),1,8,3);
        put(io," ");
      end loop;
      new_line(io);
    end loop;

    put(io,"-color");
    new_line(io);
    put(io,"cyan");
    new_line(io);

    put(io,"layer make Initial_12 ");
    new_line(io);

    for e in elem12'range loop
      put(io,"3dpoly ");
      for loc in 1 .. 2 loop
        glob := elem12(e).glob(loc);
        put(io,xo(glob),1,8,3);
        put(io,",");
        put(io,yo(glob),1,8,3);
        put(io,",");
        put(io,zo(glob),1,8,3);
        put(io," ");
      end loop;
      new_line(io);
    end loop;

    put(io,"-color");
    new_line(io);
    put(io,"white");
    new_line(io);

    put(io,"layer set 0 ");
    new_line(io);

    put(io,"zoom ext ext");
    new_line(io);

    close(io);
  end autocad;

begin
  initia;
  input;
  autocad;
end stella;
