with
  text_io,
  ada.integer_text_io,
  ada.long_float_text_io,
  ada.numerics.long_elementary_functions,
  terms,
  nlp;

procedure minshape is

  subtype fmat3x1 is terms.fmat1 (1 .. 3);

  subtype fmat2x3 is terms.fmat2 (1 .. 2, 1 .. 3);

  subtype fmat3x3 is terms.fmat2 (1 .. 3, 1 .. 3);

  subtype fmat4x3 is terms.fmat2 (1 .. 4, 1 .. 3);

  type elem11_rec is
    record
      area : long_float;
      glob : terms.imat1 (1 .. 2);
      v    : terms.fmat1 (1 .. 3);
    end record;

  type elem11_vec is array (integer range <>) of elem11_rec;

  type elem11_ptr is access elem11_vec;

  type elem12_rec is
    record
      norm   : long_float;
      lambda : long_float;
      glob   : terms.imat1 (1 .. 2);
      v      : terms.fmat1 (1 .. 3);
    end record;

  type elem12_vec is array (integer range <>) of elem12_rec;

  type elem12_ptr is access elem12_vec;

  type elem21_rec is
    record
      thick2 : long_float;
      glob   : terms.imat1 (1 .. 3);
      v1     : terms.fmat1 (1 .. 3);
      v2     : terms.fmat1 (1 .. 3);
    end record;

  type elem21_vec is array (integer range <>) of elem21_rec;

  type elem21_ptr is access elem21_vec;

  type elem22_rec is
    record
      norm   : long_float;
      lambda : long_float;
      glob   : terms.imat1 (1 .. 3);
      v1     : terms.fmat1 (1 .. 3);
      v2     : terms.fmat1 (1 .. 3);
      w      : terms.fmat1 (1 .. 3);
    end record;

  type elem22_vec is array (integer range <>) of elem22_rec;

  type elem22_ptr is access elem22_vec;

  type elem31_rec is
    record
      glob : terms.imat1 (1 .. 4);
      v1   : terms.fmat1 (1 .. 3);
      v2   : terms.fmat1 (1 .. 3);
      v3   : terms.fmat1 (1 .. 3);
    end record;

  type elem31_vec is array (integer range <>) of elem31_rec;

  type elem31_ptr is access elem31_vec;

  type elem32_rec is
    record
      vol6   : long_float;
      lambda : long_float;
      glob   : terms.imat1 (1 .. 4);
      v1     : terms.fmat1 (1 .. 3);
      v2     : terms.fmat1 (1 .. 3);
      v3     : terms.fmat1 (1 .. 3);
    end record;

  type elem32_vec is array (integer range <>) of elem32_rec;

  type elem32_ptr is access elem32_vec;

  optimal                           : boolean;
  nn,n,maxls,minor,maxminor,maxlist : integer;
  major,maxmajor                    : integer;
  lstol,f                           : long_float;
  maxnlag,maxncon,mu                : long_float;
  index                             : terms.imat2_ptr;
  xo,yo,zo,x,g                      : terms.fmat1_ptr;
  known                             : terms.fmat2_ptr;
  elem11                            : elem11_ptr;
  elem12                            : elem12_ptr;
  elem21                            : elem21_ptr;
  elem22                            : elem22_ptr;
  elem31                            : elem31_ptr;
  elem32                            : elem32_ptr;

  procedure scan (io : in text_io.file_type) is

    ascii : character;

  begin
    loop
      text_io.get(io,ascii);
      text_io.skip_line(io);
      exit when ascii = ':';
    end loop;
  end scan;

  procedure input is

    use
      text_io,
      ada.integer_text_io,
      ada.long_float_text_io;

    e,glob,axis,ne,nk : integer;
    thickness         : long_float;
    io                : file_type;

  begin
    open(io, in_file, "input.txt");

    -- The iterations terminate if the infinity norm of the gradient of the Lagrangian
    -- function becomes less than or equal to MAXNLAG and the infinity norm
    -- of the constraint vector becomes less than or equal to MAXNCON.

    scan(io);
    get(io,lstol);
    get(io,maxls);
    get(io,maxminor);
    get(io,maxlist);
    get(io,mu);
    get(io,maxnlag);
    get(io,maxncon);
    get(io,maxmajor);
    skip_line(io);

    -- nodes
    scan(io);
    get(io,nn);
    skip_line(io);

    -- initiate xo, yo, zo
    xo := new terms.fmat1 (1 .. nn);
    yo := new terms.fmat1 (1 .. nn);
    zo := new terms.fmat1 (1 .. nn);

    skip_line(io);
    for k in 1 .. nn loop
      get(io,glob);
      get(io,xo(glob));
      get(io,yo(glob));
      get(io,zo(glob));
      skip_line(io);
    end loop;

    -- elements 11
    scan(io);
    get(io,ne);
    skip_line(io);

    elem11 := new elem11_vec (1 .. ne);

    if ne > 0 then
      skip_line(io);

      for k in elem11'range loop
        get(io,e);
        get(io,elem11(e).glob(1));
        get(io,elem11(e).glob(2));
        get(io,elem11(e).area);
        skip_line(io);
      end loop;
    end if;

    -- elements 12
    scan(io);
    get(io,ne);
    skip_line(io);

    elem12 := new elem12_vec (1 .. ne);

    if ne > 0 then
      skip_line(io);

      for k in elem12'range loop
        get(io,e);
        get(io,elem12(e).glob(1));
        get(io,elem12(e).glob(2));
        skip_line(io);
      end loop;
    end if;

    -- elements 21
    scan(io);
    get(io,ne);
    skip_line(io);

    elem21 := new elem21_vec (1 .. ne);

    if ne > 0 then
      skip_line(io);

      for k in elem21'range loop
        get(io,e);
        get(io,elem21(e).glob(1));
        get(io,elem21(e).glob(2));
        get(io,elem21(e).glob(3));
        get(io,thickness);
        skip_line(io);

        elem21(e).thick2 := thickness / 2.0;
      end loop;
    end if;

    -- elements 22
    scan(io);
    get(io,ne);
    skip_line(io);

    elem22 := new elem22_vec (1 .. ne);

    if ne > 0 then
      skip_line(io);

      for k in elem22'range loop
        get(io,e);
        get(io,elem22(e).glob(1));
        get(io,elem22(e).glob(2));
        get(io,elem22(e).glob(3));
        skip_line(io);
      end loop;
    end if;

    -- elements 31
    scan(io);
    get(io,ne);
    skip_line(io);

    elem31 := new elem31_vec (1 .. ne);

    if ne > 0 then
      skip_line(io);

      for k in elem31'range loop
        get(io,e);
        get(io,elem31(e).glob(1));
        get(io,elem31(e).glob(2));
        get(io,elem31(e).glob(3));
        get(io,elem31(e).glob(4));
        skip_line(io);
      end loop;
    end if;

    -- elements 32
    scan(io);
    get(io,ne);
    skip_line(io);

    elem32 := new elem32_vec (1 .. ne);

    if ne > 0 then
      skip_line(io);

      for k in elem32'range loop
        get(io,e);
        get(io,elem32(e).glob(1));
        get(io,elem32(e).glob(2));
        get(io,elem32(e).glob(3));
        get(io,elem32(e).glob(4));
        skip_line(io);
      end loop;
    end if;

    -- initiate index
    index := new terms.imat2 (1 .. nn, 1 .. 3);
    for glob in 1 .. nn loop
      for axis in 1 .. 3 loop
        index(glob,axis) := 0;
      end loop;
    end loop;

    -- initiate known
    known := new terms.fmat2 (1 .. nn, 1 .. 3);
    for glob in 1 .. nn loop
      for axis in 1 .. 3 loop
        known(glob,axis) := 0.0;
      end loop;
    end loop;

    -- known displacements
    scan(io);
    get(io,nk);
    skip_line(io);

    if nk > 0 then
      skip_line(io);
      for k in 1 .. nk loop
        get(io,glob);
        get(io,axis);
        get(io,known(glob,axis));
        skip_line(io);
        index(glob,axis) := 1;
      end loop;
    end if;

    close(io);
  end input;

  procedure setup is

    use
      ada.numerics.long_elementary_functions;

    glob1,glob2,glob3,glob4,s1,s2 : integer;
    vol6                          : long_float;

  begin
    -- number of unknowns
    n := 3 * nn;
    for glob in 1 .. nn loop
      for axis in 1 .. 3 loop
        if index(glob,axis) = 1 then
          n := n - 1;
        end if;
      end loop;
    end loop;

    -- new sequence
    s1 := 0;
    s2 := n;
    for glob in 1 .. nn loop
      for axis in 1 .. 3 loop
        if index(glob,axis) = 0 then
          s1 := s1 + 1;
          index(glob,axis) := s1;
        else
          s2 := s2 + 1;
          index(glob,axis) := s2;
        end if;
      end loop;
    end loop;

    -- elements 11
    for e in elem11'range loop
      glob1 := elem11(e).glob(1);
      glob2 := elem11(e).glob(2);

      -- v
      elem11(e).v(1) := xo(glob2) - xo(glob1);
      elem11(e).v(2) := yo(glob2) - yo(glob1);
      elem11(e).v(3) := zo(glob2) - zo(glob1);
    end loop;

    -- elements 12
    for e in elem12'range loop
      glob1 := elem12(e).glob(1);
      glob2 := elem12(e).glob(2);

      -- v
      elem12(e).v(1) := xo(glob2) - xo(glob1);
      elem12(e).v(2) := yo(glob2) - yo(glob1);
      elem12(e).v(3) := zo(glob2) - zo(glob1);

      elem12(e).norm := sqrt(elem12(e).v(1) ** 2 + elem12(e).v(2) ** 2 + elem12(e).v(3) ** 2);
    end loop;

    -- elements 21
    for e in elem21'range loop
      glob1 := elem21(e).glob(1);
      glob2 := elem21(e).glob(2);
      glob3 := elem21(e).glob(3);

      -- v1
      elem21(e).v1(1) := xo(glob3) - xo(glob2);
      elem21(e).v1(2) := yo(glob3) - yo(glob2);
      elem21(e).v1(3) := zo(glob3) - zo(glob2);

      -- v2
      elem21(e).v2(1) := xo(glob1) - xo(glob3);
      elem21(e).v2(2) := yo(glob1) - yo(glob3);
      elem21(e).v2(3) := zo(glob1) - zo(glob3);
    end loop;

    -- elements 22
    for e in elem22'range loop
      glob1 := elem22(e).glob(1);
      glob2 := elem22(e).glob(2);
      glob3 := elem22(e).glob(3);

      -- v1
      elem22(e).v1(1) := xo(glob3) - xo(glob2);
      elem22(e).v1(2) := yo(glob3) - yo(glob2);
      elem22(e).v1(3) := zo(glob3) - zo(glob2);

      -- v2
      elem22(e).v2(1) := xo(glob1) - xo(glob3);
      elem22(e).v2(2) := yo(glob1) - yo(glob3);
      elem22(e).v2(3) := zo(glob1) - zo(glob3);

      -- w = v1 * v2
      elem22(e).w(1) := elem22(e).v1(2) * elem22(e).v2(3) - elem22(e).v1(3) * elem22(e).v2(2);
      elem22(e).w(2) := elem22(e).v1(3) * elem22(e).v2(1) - elem22(e).v1(1) * elem22(e).v2(3);
      elem22(e).w(3) := elem22(e).v1(1) * elem22(e).v2(2) - elem22(e).v1(2) * elem22(e).v2(1);

      elem22(e).norm := sqrt(elem22(e).w(1) ** 2 + elem22(e).w(2) ** 2 + elem22(e).w(3) ** 2);
    end loop;

    -- elements 31
    for e in elem31'range loop
      glob1 := elem31(e).glob(1);
      glob2 := elem31(e).glob(2);
      glob3 := elem31(e).glob(3);
      glob4 := elem31(e).glob(4);

      -- v1
      elem31(e).v1(1) := xo(glob1) - xo(glob4);
      elem31(e).v1(2) := yo(glob1) - yo(glob4);
      elem31(e).v1(3) := zo(glob1) - zo(glob4);

      -- v2
      elem31(e).v2(1) := xo(glob2) - xo(glob4);
      elem31(e).v2(2) := yo(glob2) - yo(glob4);
      elem31(e).v2(3) := zo(glob2) - zo(glob4);

      -- v3
      elem31(e).v3(1) := xo(glob3) - xo(glob4);
      elem31(e).v3(2) := yo(glob3) - yo(glob4);
      elem31(e).v3(3) := zo(glob3) - zo(glob4);
    end loop;

    -- elements 32
    for e in elem32'range loop
      glob1 := elem32(e).glob(1);
      glob2 := elem32(e).glob(2);
      glob3 := elem32(e).glob(3);
      glob4 := elem32(e).glob(4);

      -- v1
      elem32(e).v1(1) := xo(glob1) - xo(glob4);
      elem32(e).v1(2) := yo(glob1) - yo(glob4);
      elem32(e).v1(3) := zo(glob1) - zo(glob4);

      -- v2
      elem32(e).v2(1) := xo(glob2) - xo(glob4);
      elem32(e).v2(2) := yo(glob2) - yo(glob4);
      elem32(e).v2(3) := zo(glob2) - zo(glob4);

      -- v3
      elem32(e).v3(1) := xo(glob3) - xo(glob4);
      elem32(e).v3(2) := yo(glob3) - yo(glob4);
      elem32(e).v3(3) := zo(glob3) - zo(glob4);

      -- volume 6
      vol6 := 0.0;
      vol6 := vol6 + elem32(e).v1(1) * (elem32(e).v2(2) * elem32(e).v3(3) - elem32(e).v2(3) * elem32(e).v3(2));
      vol6 := vol6 + elem32(e).v1(2) * (elem32(e).v2(3) * elem32(e).v3(1) - elem32(e).v2(1) * elem32(e).v3(3));
      vol6 := vol6 + elem32(e).v1(3) * (elem32(e).v2(1) * elem32(e).v3(2) - elem32(e).v2(2) * elem32(e).v3(1));

      elem32(e).vol6 := vol6;
    end loop;

    -- initiate x
    x := new terms.fmat1 (1 .. n);
    for k in 1 .. n loop
      x(k) := 0.0;
    end loop;

    -- initiate g
    g := new terms.fmat1 (1 .. n);
    for k in 1 .. n loop
      g(k) := 0.0;
    end loop;

    -- initiate lambda
    for e in elem12'range loop
      elem12(e).lambda := 0.0;
    end loop;

    for e in elem22'range loop
      elem22(e).lambda := 0.0;
    end loop;

    for e in elem32'range loop
      elem32(e).lambda := 0.0;
    end loop;
  end setup;

  procedure element11 (elem11 : in elem11_rec;
                       fe     : out long_float;
                       x      : in terms.fmat1_ptr;
                       ge     : out fmat2x3) is

    use
      ada.numerics.long_elementary_functions;

    glob,i   : integer;
    norm,aux : long_float;
    v        : terms.fmat1 (1 .. 3);
    xe       : terms.fmat2 (1 .. 2, 1 .. 3);

  begin
    -- global displacements
    for loc in 1 .. 2 loop
      glob := elem11.glob(loc);
      for axis in 1 .. 3 loop
        i := index(glob,axis);
        if i > n then
          xe(loc,axis) := known(glob,axis);
        else
          xe(loc,axis) := x(i);
        end if;
      end loop;
    end loop;

    -- v
    for axis in 1 .. 3 loop
      v(axis) := elem11.v(axis) + xe(2,axis) - xe(1,axis);
    end loop;
    norm := sqrt(v(1) ** 2 + v(2) ** 2 + v(3) ** 2);

    -- objective
    fe := elem11.area * norm;

    -- gradient
    aux := elem11.area / norm;
    for axis in 1 .. 3 loop
      v(axis) := aux * v(axis);
    end loop;

    for axis in 1 ..3 loop
      ge(1,axis) := -v(axis);
      ge(2,axis) := v(axis);
    end loop;
  end element11;

  procedure element12 (elem12 : in elem12_rec;
                       fe     : out long_float;
                       x      : in terms.fmat1_ptr;
                       ge     : out fmat2x3) is

    use
      ada.numerics.long_elementary_functions;

    glob,i   : integer;
    norm,eps : long_float;
    v,z      : terms.fmat1 (1 .. 3);
    xe       : terms.fmat2 (1 .. 2, 1 .. 3);

  begin
    -- global displacements
    for loc in 1 .. 2 loop
      glob := elem12.glob(loc);
      for axis in 1 .. 3 loop
        i := index(glob,axis);
        if i > n then
          xe(loc,axis) := known(glob,axis);
        else
          xe(loc,axis) := x(i);
        end if;
      end loop;
    end loop;

    -- z
    for axis in 1 .. 3 loop
      z(axis) := xe(2,axis) - xe(1,axis);
    end loop;

    -- eps
    eps := 0.0;
    for axis in 1 .. 3 loop
      eps := eps + z(axis) * (elem12.v(axis) + elem12.v(axis) + z(axis));
    end loop;
    eps := eps / elem12.norm ** 2;
    eps := eps / (sqrt(1.0 + eps) + 1.0);

    -- objective
    fe := eps;

    -- v
    for axis in 1 .. 3 loop
      v(axis) := elem12.v(axis) + z(axis);
    end loop;
    norm := sqrt(v(1) ** 2 + v(2) ** 2 + v(3) ** 2);

    -- gradient
    eps := 1.0 / elem12.norm / norm;
    for axis in 1 .. 3 loop
      v(axis) := eps * v(axis);
    end loop;

    for axis in 1 ..3 loop
      ge(1,axis) := -v(axis);
      ge(2,axis) := v(axis);
    end loop;
  end element12;

  procedure element21 (elem21 : in elem21_rec;
                       fe     : out long_float;
                       x      : in terms.fmat1_ptr;
                       ge     : out fmat3x3) is

    use
      ada.numerics.long_elementary_functions;

    glob,i   : integer;
    norm,aux : long_float;
    v1,v2,w  : terms.fmat1 (1 .. 3);
    xe       : terms.fmat2 (1 .. 3, 1 .. 3);

  begin
    -- global displacements
    for loc in 1 .. 3 loop
      glob := elem21.glob(loc);
      for axis in 1 .. 3 loop
        i := index(glob,axis);
        if i > n then
          xe(loc,axis) := known(glob,axis);
        else
          xe(loc,axis) := x(i);
        end if;
      end loop;
    end loop;

    -- v1, v2
    for axis in 1 .. 3 loop
      v1(axis) := elem21.v1(axis) + xe(3,axis) - xe(2,axis);
      v2(axis) := elem21.v2(axis) + xe(1,axis) - xe(3,axis);
    end loop;

    -- w = v1 * v2
    w(1) := v1(2) * v2(3) - v1(3) * v2(2);
    w(2) := v1(3) * v2(1) - v1(1) * v2(3);
    w(3) := v1(1) * v2(2) - v1(2) * v2(1);
    norm := sqrt(w(1) ** 2 + w(2) ** 2 + w(3) ** 2);

    -- objective
    fe := elem21.thick2 * norm;

    -- gradient
    aux := elem21.thick2 / norm;
    for axis in 1 .. 3 loop
      w(axis) := aux * w(axis);
    end loop;

    ge(1,1) := w(2) * v1(3) - w(3) * v1(2);
    ge(1,2) := w(3) * v1(1) - w(1) * v1(3);
    ge(1,3) := w(1) * v1(2) - w(2) * v1(1);

    ge(2,1) := w(2) * v2(3) - w(3) * v2(2);
    ge(2,2) := w(3) * v2(1) - w(1) * v2(3);
    ge(2,3) := w(1) * v2(2) - w(2) * v2(1);

    ge(3,1) := -(ge(1,1) + ge(2,1));
    ge(3,2) := -(ge(1,2) + ge(2,2));
    ge(3,3) := -(ge(1,3) + ge(2,3));
  end element21;

  procedure element22 (elem22 : in elem22_rec;
                       fe     : out long_float;
                       x      : in terms.fmat1_ptr;
                       ge     : out fmat3x3) is

    use
      ada.numerics.long_elementary_functions;

    glob,i      : integer;
    norm,eps    : long_float;
    v1,v2,w,z,s : terms.fmat1 (1 .. 3);
    xe          : terms.fmat2 (1 .. 3, 1 .. 3);

  begin
    -- global displacements
    for loc in 1 .. 3 loop
      glob := elem22.glob(loc);
      for axis in 1 .. 3 loop
        i := index(glob,axis);
        if i > n then
          xe(loc,axis) := known(glob,axis);
        else
          xe(loc,axis) := x(i);
        end if;
      end loop;
    end loop;

    -- z = 0
    z(1) := 0.0;
    z(2) := 0.0;
    z(3) := 0.0;

    -- z = z + (v1 - x2) * x1
    s(1) := elem22.v1(1) - xe(2,1);
    s(2) := elem22.v1(2) - xe(2,2);
    s(3) := elem22.v1(3) - xe(2,3);
    z(1) := z(1) + s(2) * xe(1,3) - s(3) * xe(1,2);
    z(2) := z(2) + s(3) * xe(1,1) - s(1) * xe(1,3);
    z(3) := z(3) + s(1) * xe(1,2) - s(2) * xe(1,1);

    -- z = z + (v2 - x3) * x2
    s(1) := elem22.v2(1) - xe(3,1);
    s(2) := elem22.v2(2) - xe(3,2);
    s(3) := elem22.v2(3) - xe(3,3);
    z(1) := z(1) + s(2) * xe(2,3) - s(3) * xe(2,2);
    z(2) := z(2) + s(3) * xe(2,1) - s(1) * xe(2,3);
    z(3) := z(3) + s(1) * xe(2,2) - s(2) * xe(2,1);

    -- z = z + x3 * (v1 + v2 + x1)
    s(1) := elem22.v1(1) + elem22.v2(1) + xe(1,1);
    s(2) := elem22.v1(2) + elem22.v2(2) + xe(1,2);
    s(3) := elem22.v1(3) + elem22.v2(3) + xe(1,3);
    z(1) := z(1) + xe(3,2) * s(3) - xe(3,3) * s(2);
    z(2) := z(2) + xe(3,3) * s(1) - xe(3,1) * s(3);
    z(3) := z(3) + xe(3,1) * s(2) - xe(3,2) * s(1);

    -- eps
    eps := 0.0;
    for axis in 1 .. 3 loop
      eps := eps + z(axis) * (elem22.w(axis) + elem22.w(axis) + z(axis));
    end loop;
    eps := eps / elem22.norm ** 2;
    eps := eps / (sqrt(1.0 + eps) + 1.0);

    -- objective
    fe := eps;

    -- v1, v2
    for axis in 1 .. 3 loop
      v1(axis) := elem22.v1(axis) + (xe(3,axis) - xe(2,axis));
      v2(axis) := elem22.v2(axis) + (xe(1,axis) - xe(3,axis));
    end loop;

    -- w = v1 * v2
    w(1) := v1(2) * v2(3) - v1(3) * v2(2);
    w(2) := v1(3) * v2(1) - v1(1) * v2(3);
    w(3) := v1(1) * v2(2) - v1(2) * v2(1);
    norm := sqrt(w(1) ** 2 + w(2) ** 2 + w(3) ** 2);

    -- gradient
    eps := 1.0 / elem22.norm / norm;
    for axis in 1 .. 3 loop
      w(axis) := eps * w(axis);
    end loop;

    -- w * v1
    ge(1,1) := w(2) * v1(3) - w(3) * v1(2);
    ge(1,2) := w(3) * v1(1) - w(1) * v1(3);
    ge(1,3) := w(1) * v1(2) - w(2) * v1(1);

    -- w * v2
    ge(2,1) := w(2) * v2(3) - w(3) * v2(2);
    ge(2,2) := w(3) * v2(1) - w(1) * v2(3);
    ge(2,3) := w(1) * v2(2) - w(2) * v2(1);

    -- -(w * v1 + w * v2)
    ge(3,1) := -(ge(1,1) + ge(2,1));
    ge(3,2) := -(ge(1,2) + ge(2,2));
    ge(3,3) := -(ge(1,3) + ge(2,3));
  end element22;

  procedure element31 (elem31 : in elem31_rec;
                       fe     : out long_float;
                       x      : in terms.fmat1_ptr;
                       ge     : out fmat4x3) is

    glob,i   : integer;
    vol      : long_float;
    v1,v2,v3 : terms.fmat1 (1 .. 3);
    w1,w2,w3 : terms.fmat1 (1 .. 3);
    xe       : terms.fmat2 (1 .. 4, 1 .. 3);

  begin
    -- global displacements
    for loc in 1 .. 4 loop
      glob := elem31.glob(loc);
      for axis in 1 .. 3 loop
        i := index(glob,axis);
        if i > n then
          xe(loc,axis) := known(glob,axis);
        else
          xe(loc,axis) := x(i);
        end if;
      end loop;
    end loop;

    -- v1, v2, v3
    for axis in 1 .. 3 loop
      v1(axis) := elem31.v1(axis) + (xe(1,axis) - xe(4,axis));
      v2(axis) := elem31.v2(axis) + (xe(2,axis) - xe(4,axis));
      v3(axis) := elem31.v3(axis) + (xe(3,axis) - xe(4,axis));
    end loop;

    -- (v2 x v3) / 6
    w1(1) := (v2(2) * v3(3) - v2(3) * v3(2)) / 6.0;
    w1(2) := (v2(3) * v3(1) - v2(1) * v3(3)) / 6.0;
    w1(3) := (v2(1) * v3(2) - v2(2) * v3(1)) / 6.0;

    -- (v3 x v1 / 6
    w2(1) := (v3(2) * v1(3) - v3(3) * v1(2)) / 6.0;
    w2(2) := (v3(3) * v1(1) - v3(1) * v1(3)) / 6.0;
    w2(3) := (v3(1) * v1(2) - v3(2) * v1(1)) / 6.0;

    -- (v1 x v2 / 6
    w3(1) := (v1(2) * v2(3) - v1(3) * v2(2)) / 6.0;
    w3(2) := (v1(3) * v2(1) - v1(1) * v2(3)) / 6.0;
    w3(3) := (v1(1) * v2(2) - v1(2) * v2(1)) / 6.0;

    -- vol
    vol := v1(1) * w1(1) + v1(2) * w1(2) + v1(3) * w1(3);

    -- objective
    fe := vol;

    -- gradient
    for axis in 1 .. 3 loop
      ge(1,axis) := w1(axis);
      ge(2,axis) := w2(axis);
      ge(3,axis) := w3(axis);
      ge(4,axis) := -(w1(axis) + w2(axis) + w3(axis));
    end loop;
  end element31;

  procedure element32 (elem32 : in elem32_rec;
                       fe     : out long_float;
                       x      : in terms.fmat1_ptr;
                       ge     : out fmat4x3) is

    glob,i            : integer;
    vol6,eps          : long_float;
    z1,z2,z3,v1,v2,v3 : terms.fmat1 (1 .. 3);
    w1,w2,w3          : terms.fmat1 (1 .. 3);
    xe                : terms.fmat2 (1 .. 4, 1 .. 3);

  begin
    -- global displacements
    for loc in 1 .. 4 loop
      glob := elem32.glob(loc);
      for axis in 1 .. 3 loop
        i := index(glob,axis);
        if i > n then
          xe(loc,axis) := known(glob,axis);
        else
          xe(loc,axis) := x(i);
        end if;
      end loop;
    end loop;

    -- z1, z2, z3
    for axis in 1 .. 3 loop
      z1(axis) := xe(1,axis) - xe(4,axis);
      z2(axis) := xe(2,axis) - xe(4,axis);
      z3(axis) := xe(3,axis) - xe(4,axis);
    end loop;

    -- v1, v2, v3
    for axis in 1 .. 3 loop
      v1(axis) := elem32.v1(axis) + z1(axis);
      v2(axis) := elem32.v2(axis) + z2(axis);
      v3(axis) := elem32.v3(axis) + z3(axis);
    end loop;

    -- v2 x v3
    w1(1) := v2(2) * v3(3) - v2(3) * v3(2);
    w1(2) := v2(3) * v3(1) - v2(1) * v3(3);
    w1(3) := v2(1) * v3(2) - v2(2) * v3(1);

    -- v3 x v1
    w2(1) := v3(2) * v1(3) - v3(3) * v1(2);
    w2(2) := v3(3) * v1(1) - v3(1) * v1(3);
    w2(3) := v3(1) * v1(2) - v3(2) * v1(1);

    -- v1 x v2
    w3(1) := v1(2) * v2(3) - v1(3) * v2(2);
    w3(2) := v1(3) * v2(1) - v1(1) * v2(3);
    w3(3) := v1(1) * v2(2) - v1(2) * v2(1);

    -- vol6
    vol6 := v1(1) * w1(1) + v1(2) * w1(2) + v1(3) * w1(3);

    -- eps
    eps := 0.0;

    eps := eps + elem32.v1(1) * (v2(2) * z3(3) - v2(3) * z3(2));
    eps := eps + elem32.v1(2) * (v2(3) * z3(1) - v2(1) * z3(3));
    eps := eps + elem32.v1(3) * (v2(1) * z3(2) - v2(2) * z3(1));

    eps := eps + elem32.v2(1) * (v3(2) * z1(3) - v3(3) * z1(2));
    eps := eps + elem32.v2(2) * (v3(3) * z1(1) - v3(1) * z1(3));
    eps := eps + elem32.v2(3) * (v3(1) * z1(2) - v3(2) * z1(1));

    eps := eps + elem32.v3(1) * (v1(2) * z2(3) - v1(3) * z2(2));
    eps := eps + elem32.v3(2) * (v1(3) * z2(1) - v1(1) * z2(3));
    eps := eps + elem32.v3(3) * (v1(1) * z2(2) - v1(2) * z2(1));

    eps := eps + z1(1) * (z2(2) * z3(3) - z2(3) * z3(2));
    eps := eps + z1(2) * (z2(3) * z3(1) - z2(1) * z3(3));
    eps := eps + z1(3) * (z2(1) * z3(2) - z2(2) * z3(1));

    eps := eps / elem32.vol6;

    -- objective
    fe := eps;

    -- gradient
    eps := 1.0 / elem32.vol6;
    for axis in 1 .. 3 loop
      ge(1,axis) := eps * w1(axis);
      ge(2,axis) := eps * w2(axis);
      ge(3,axis) := eps * w3(axis);
      ge(4,axis) := -eps * (w1(axis) + w2(axis) + w3(axis));
    end loop;
  end element32;

  procedure objective (n : in integer;
                       f : out long_float;
                       x : in terms.fmat1_ptr;
                       g : in terms.fmat1_ptr) is

    glob,i : integer;
    fe,aux : long_float;
    ge1    : fmat2x3;
    ge2    : fmat3x3;
    ge3    : fmat4x3;

  begin
    -- initiate objective, gradient
    f := 0.0;
    for i in 1 .. n loop
      g(i) := 0.0;
    end loop;

    -- element 11
    for e in elem11'range loop
      element11(elem11(e),fe,x,ge1);

      -- update objective
      f := f + fe;

      -- update gradient
      for loc in 1 .. 2 loop
        glob := elem11(e).glob(loc);
        for axis in 1 .. 3 loop
          i := index(glob,axis);
          if i <= n then
            g(i) := g(i) + ge1(loc,axis);
          end if;
        end loop;
      end loop;
    end loop;

    -- element 12
    for e in elem12'range loop
      element12(elem12(e),fe,x,ge1);

      -- update objective
      f := f + (elem12(e).lambda + fe / mu / 2.0) * fe;

      -- update gradient
      aux := elem12(e).lambda + fe / mu;
      for loc in 1 .. 2 loop
        glob := elem12(e).glob(loc);
        for axis in 1 .. 3 loop
          i := index(glob,axis);
          if i <= n then
            g(i) := g(i) + aux * ge1(loc,axis);
          end if;
        end loop;
      end loop;
    end loop;

    -- element 21
    for e in elem21'range loop
      element21(elem21(e),fe,x,ge2);

      -- update objective
      f := f + fe;

      -- update gradient
      for loc in 1 .. 3 loop
        glob := elem21(e).glob(loc);
        for axis in 1 .. 3 loop
          i := index(glob,axis);
          if i <= n then
            g(i) := g(i) + ge2(loc,axis);
          end if;
        end loop;
      end loop;
    end loop;

    -- element 22
    for e in elem22'range loop
      element22(elem22(e),fe,x,ge2);

      -- update objective
      f := f + (elem22(e).lambda + fe / mu / 2.0) * fe;

      -- update gradient
      aux := elem22(e).lambda + fe / mu;
      for loc in 1 .. 3 loop
        glob := elem22(e).glob(loc);
        for axis in 1 .. 3 loop
          i := index(glob,axis);
          if i <= n then
            g(i) := g(i) + aux * ge2(loc,axis);
          end if;
        end loop;
      end loop;
    end loop;

    -- element 31
    for e in elem31'range loop
      element31(elem31(e),fe,x,ge3);

      -- update objective
      f := f + fe;

      -- update gradient
      for loc in 1 .. 4 loop
        glob := elem31(e).glob(loc);
        for axis in 1 .. 3 loop
          i := index(glob,axis);
          if i <= n then
            g(i) := g(i) + ge3(loc,axis);
          end if;
        end loop;
      end loop;
    end loop;

    -- element 32
    for e in elem32'range loop
      element32(elem32(e),fe,x,ge3);

      -- update objective
      f := f + (elem32(e).lambda + fe / mu / 2.0) * fe;

      -- update gradient
      aux := elem32(e).lambda + fe / mu;
      for loc in 1 .. 4 loop
        glob := elem32(e).glob(loc);
        for axis in 1 .. 3 loop
          i := index(glob,axis);
          if i <= n then
            g(i) := g(i) + aux * ge3(loc,axis);
          end if;
        end loop;
      end loop;
    end loop;
  end objective;

  package qn is new nlp.qnewton(objective);

  function ncon (x : in terms.fmat1_ptr) return long_float is

    fe,norm : long_float;
    ge1     : fmat2x3;
    ge2     : fmat3x3;
    ge3     : fmat4x3;

  begin
    norm := 0.0;

    -- element 12
    for e in elem12'range loop
      element12(elem12(e),fe,x,ge1);

      if abs(fe) > norm then
        norm := abs(fe);
      end if;
    end loop;

    -- element 22
    for e in elem22'range loop
      element22(elem22(e),fe,x,ge2);

      if abs(fe) > norm then
        norm := abs(fe);
      end if;
    end loop;

    -- element 32
    for e in elem32'range loop
      element32(elem32(e),fe,x,ge3);

      if abs(fe) > norm then
        norm := abs(fe);
      end if;
    end loop;

    return norm;
  end ncon;

  function nlag (x : in terms.fmat1_ptr) return long_float is

    glob,i  : integer;
    fe,norm : long_float;
    ge1     : fmat2x3;
    ge2     : fmat3x3;
    ge3     : fmat4x3;

  begin
    -- initiate gradient
    for i in 1 .. n loop
      g(i) := 0.0;
    end loop;

    -- element 11
    for e in elem11'range loop
      element11(elem11(e),fe,x,ge1);

      -- update gradient
      for loc in 1 .. 2 loop
        glob := elem11(e).glob(loc);
        for axis in 1 .. 3 loop
          i := index(glob,axis);
          if i <= n then
            g(i) := g(i) + ge1(loc,axis);
          end if;
        end loop;
      end loop;
    end loop;

    -- element 12
    for e in elem12'range loop
      element12(elem12(e),fe,x,ge1);

      -- update gradient
      norm := elem12(e).lambda;
      for loc in 1 .. 2 loop
        glob := elem12(e).glob(loc);
        for axis in 1 .. 3 loop
          i := index(glob,axis);
          if i <= n then
            g(i) := g(i) + norm * ge1(loc,axis);
          end if;
        end loop;
      end loop;
    end loop;

    -- element 21
    for e in elem21'range loop
      element21(elem21(e),fe,x,ge2);

      -- update gradient
      for loc in 1 .. 3 loop
        glob := elem21(e).glob(loc);
        for axis in 1 .. 3 loop
          i := index(glob,axis);
          if i <= n then
            g(i) := g(i) + ge2(loc,axis);
          end if;
        end loop;
      end loop;
    end loop;

    -- element 22
    for e in elem22'range loop
      element22(elem22(e),fe,x,ge2);

      -- update gradient
      norm := elem22(e).lambda;
      for loc in 1 .. 3 loop
        glob := elem22(e).glob(loc);
        for axis in 1 .. 3 loop
          i := index(glob,axis);
          if i <= n then
            g(i) := g(i) + norm * ge2(loc,axis);
          end if;
        end loop;
      end loop;
    end loop;

    -- element 31
    for e in elem31'range loop
      element31(elem31(e),fe,x,ge3);

      -- update gradient
      for loc in 1 .. 4 loop
        glob := elem31(e).glob(loc);
        for axis in 1 .. 3 loop
          i := index(glob,axis);
          if i <= n then
            g(i) := g(i) + ge3(loc,axis);
          end if;
        end loop;
      end loop;
    end loop;

    -- element 32
    for e in elem32'range loop
      element32(elem32(e),fe,x,ge3);

      -- update gradient
      norm := elem32(e).lambda;
      for loc in 1 .. 4 loop
        glob := elem32(e).glob(loc);
        for axis in 1 .. 3 loop
          i := index(glob,axis);
          if i <= n then
            g(i) := g(i) + norm * ge3(loc,axis);
          end if;
        end loop;
      end loop;
    end loop;

    norm := 0.0;
    for i in 1 .. n loop
      if abs(g(i)) > norm then
        norm := abs(g(i));
      end if;
    end loop;

    return norm;
  end nlag;

  procedure updatelag (x : in terms.fmat1_ptr) is

    fe  : long_float;
    ge1 : fmat2x3;
    ge2 : fmat3x3;
    ge3 : fmat4x3;

  begin
    -- element 12
    for e in elem12'range loop
      element12(elem12(e),fe,x,ge1);
      elem12(e).lambda := elem12(e).lambda + fe / mu;
    end loop;

    -- element 22
    for e in elem22'range loop
      element22(elem22(e),fe,x,ge2);
      elem22(e).lambda := elem22(e).lambda + fe / mu;
    end loop;

    -- element 32
    for e in elem32'range loop
      element32(elem32(e),fe,x,ge3);
      elem32(e).lambda := elem32(e).lambda + fe / mu;
    end loop;
  end updatelag;

  procedure auglag is

    use
      ada.numerics.long_elementary_functions;

    tau,omega0,eta0,gamma1,alpha,omega,eta : long_float;

    function min (a : in long_float; b : in long_float) return long_float is

    begin
      if a < b then
        return a;
      else
        return b;
      end if;
    end min;

    function max (a : in long_float; b : in long_float) return long_float is

    begin
      if a > b then
        return a;
      else
        return b;
      end if;
    end max;

  begin
    -- tau < 1
    tau := 0.1;
    omega0 := 1.0;
    eta0 := 1.0;
    gamma1 := 0.1;

    alpha := min(mu,gamma1);
    omega := max(maxnlag,omega0 * alpha);
    eta := max(maxncon,eta0 * exp(0.1 * log(alpha)));
    major := 0;

    while ((nlag(x) > maxnlag) or (ncon(x)) > maxncon) and (major < maxmajor) loop
      major := major + 1;

      qn.lbfgs(maxlist,n,maxminor,maxls,
               minor,
               omega,lstol,
               f,
               x,g,
               optimal);

      if ncon(x) <= eta then
        updatelag(x);
        alpha := min(mu,gamma1);
        omega := max(maxnlag,omega * alpha);
        eta := max(maxncon,eta * exp(0.9 * log(alpha)));
      else
        mu := tau * mu;
        alpha := min(mu,gamma1);
        omega := max(maxnlag,omega0 * alpha);
        eta := max(maxncon,eta0 * exp(0.1 * log(alpha)));
      end if;
    end loop;
  end auglag;

  procedure vertex (glob : in integer;
                    vx   : out fmat3x1) is

    i : integer;

  begin
    vx(1) := xo(glob);
    vx(2) := yo(glob);
    vx(3) := zo(glob);

    for axis in 1 .. 3 loop
      i := index(glob,axis);
      if i <= n then
        vx(axis) := vx(axis) + x(i);
      else
        vx(axis) := vx(axis) + known(glob,axis);
      end if;
    end loop;
  end vertex;

  procedure autocad is

    use
      text_io,
      ada.integer_text_io,
      ada.long_float_text_io;

    glob : integer;
    vx   : fmat3x1;
    io   : file_type;

  begin
    create(io, out_file, "autocad.scr");

    -- set running object snap mode to NONE
    put(io,"osmode 0");
    new_line(io);

    -- element 31
    if elem31'last > 0 then
      put(io,"-color");
      new_line(io);
      put(io,"blue");
      new_line(io);

      put(io,"layer make Initial_31 ");
      new_line(io);

      for e in elem31'range loop
        put(io,"pface ");
        for loc in 1 .. 4 loop
          glob := elem31(e).glob(loc);
          put(io,xo(glob),1,6,3);
          put(io,",");
          put(io,yo(glob),1,6,3);
          put(io,",");
          put(io,zo(glob),1,6,3);
          put(io," ");
        end loop;
        new_line(io);
        put(io,"1 2 3 ");
        new_line(io);
        put(io,"1 2 4 ");
        new_line(io);
        put(io,"2 3 4 ");
        new_line(io);
        put(io,"3 1 4 ");
        put(io," ");
        new_line(io);
      end loop;

      put(io,"layer make Final_31 ");
      new_line(io);

      for e in elem31'range loop
        put(io,"pface ");
        for loc in 1 .. 4 loop
          glob := elem31(e).glob(loc);
          vertex(glob,vx);
          put(io,vx(1),1,6,3);
          put(io,",");
          put(io,vx(2),1,6,3);
          put(io,",");
          put(io,vx(3),1,6,3);
          put(io," ");
        end loop;
        new_line(io);
        put(io,"1 2 3 ");
        new_line(io);
        put(io,"1 2 4 ");
        new_line(io);
        put(io,"2 3 4 ");
        new_line(io);
        put(io,"3 1 4 ");
        put(io," ");
        new_line(io);
      end loop;

      put(io,"layer off Initial_31 ");
      new_line(io);
    end if;

    -- element 32
    if elem32'last > 0 then
      put(io,"-color");
      new_line(io);
      put(io,"yellow");
      new_line(io);

      put(io,"layer make Initial_32 ");
      new_line(io);

      for e in elem32'range loop
        put(io,"pface ");
        for loc in 1 .. 4 loop
          glob := elem32(e).glob(loc);
          put(io,xo(glob),1,6,3);
          put(io,",");
          put(io,yo(glob),1,6,3);
          put(io,",");
          put(io,zo(glob),1,6,3);
          put(io," ");
        end loop;
        new_line(io);
        put(io,"1 2 3 ");
        new_line(io);
        put(io,"1 2 4 ");
        new_line(io);
        put(io,"2 3 4 ");
        new_line(io);
        put(io,"3 1 4 ");
        put(io," ");
        new_line(io);
      end loop;

      put(io,"layer make Final_32 ");
      new_line(io);

      for e in elem32'range loop
        put(io,"pface ");
        for loc in 1 .. 4 loop
          glob := elem32(e).glob(loc);
          vertex(glob,vx);
          put(io,vx(1),1,6,3);
          put(io,",");
          put(io,vx(2),1,6,3);
          put(io,",");
          put(io,vx(3),1,6,3);
          put(io," ");
        end loop;
        new_line(io);
        put(io,"1 2 3 ");
        new_line(io);
        put(io,"1 2 4 ");
        new_line(io);
        put(io,"2 3 4 ");
        new_line(io);
        put(io,"3 1 4 ");
        put(io," ");
        new_line(io);
      end loop;

      put(io,"layer off Initial_32 ");
      new_line(io);
    end if;

    -- element 21
    if elem21'last > 0 then
      put(io,"-color");
      new_line(io);
      put(io,"green");
      new_line(io);

      put(io,"layer make Initial_21 ");
      new_line(io);

      for e in elem21'range loop
        put(io,"3dface ");
        for loc in 1 .. 3 loop
          glob := elem21(e).glob(loc);
          put(io,xo(glob),1,6,3);
          put(io,",");
          put(io,yo(glob),1,6,3);
          put(io,",");
          put(io,zo(glob),1,6,3);
          put(io," ");
        end loop;
        put(io," ");
        new_line(io);
      end loop;

      put(io,"layer make Final_21 ");
      new_line(io);

      for e in elem21'range loop
        put(io,"3dface ");
        for loc in 1 .. 3 loop
          glob := elem21(e).glob(loc);
          vertex(glob,vx);
          put(io,vx(1),1,6,3);
          put(io,",");
          put(io,vx(2),1,6,3);
          put(io,",");
          put(io,vx(3),1,6,3);
          put(io," ");
        end loop;
        put(io," ");
        new_line(io);
      end loop;

      put(io,"layer off Initial_21 ");
      new_line(io);
    end if;

    -- element 22
    if elem22'last > 0 then
      put(io,"-color");
      new_line(io);
      put(io,"magenta");
      new_line(io);

      put(io,"layer make Initial_22 ");
      new_line(io);

      for e in elem22'range loop
        put(io,"3dface ");
        for loc in 1 .. 3 loop
          glob := elem22(e).glob(loc);
          put(io,xo(glob),1,6,3);
          put(io,",");
          put(io,yo(glob),1,6,3);
          put(io,",");
          put(io,zo(glob),1,6,3);
          put(io," ");
        end loop;
        put(io," ");
        new_line(io);
      end loop;

      put(io,"layer make Final_22 ");
      new_line(io);

      for e in elem22'range loop
        put(io,"3dface ");
        for loc in 1 .. 3 loop
          glob := elem22(e).glob(loc);
          vertex(glob,vx);
          put(io,vx(1),1,6,3);
          put(io,",");
          put(io,vx(2),1,6,3);
          put(io,",");
          put(io,vx(3),1,6,3);
          put(io," ");
        end loop;
        put(io," ");
        new_line(io);
      end loop;

      put(io,"layer off Initial_22 ");
      new_line(io);
    end if;

    -- element 11
    if elem11'last > 0 then
      put(io,"-color");
      new_line(io);
      put(io,"red");
      new_line(io);

      put(io,"layer make Initial_11 ");
      new_line(io);

      for e in elem11'range loop
        put(io,"3dpoly ");
        for loc in 1 .. 2 loop
          glob := elem11(e).glob(loc);
          put(io,xo(glob),1,6,3);
          put(io,",");
          put(io,yo(glob),1,6,3);
          put(io,",");
          put(io,zo(glob),1,6,3);
          put(io," ");
        end loop;
        new_line(io);
      end loop;

      put(io,"layer make Final_11 ");
      new_line(io);

      for e in elem11'range loop
        put(io,"3dpoly ");
        for loc in 1 .. 2 loop
          glob := elem11(e).glob(loc);
          vertex(glob,vx);
          put(io,vx(1),1,6,3);
          put(io,",");
          put(io,vx(2),1,6,3);
          put(io,",");
          put(io,vx(3),1,6,3);
          put(io," ");
        end loop;
        new_line(io);
      end loop;

      put(io,"layer off Initial_11 ");
      new_line(io);
    end if;

    -- element 12
    if elem12'last > 0 then
      put(io,"-color");
      new_line(io);
      put(io,"cyan");
      new_line(io);

      put(io,"layer make Initial_12 ");
      new_line(io);

      for e in elem12'range loop
        put(io,"3dpoly ");
        for loc in 1 .. 2 loop
          glob := elem12(e).glob(loc);
          put(io,xo(glob),1,6,3);
          put(io,",");
          put(io,yo(glob),1,6,3);
          put(io,",");
          put(io,zo(glob),1,6,3);
          put(io," ");
        end loop;
        new_line(io);
      end loop;

      put(io,"layer make Final_12 ");
      new_line(io);

      for e in elem12'range loop
        put(io,"3dpoly ");
        for loc in 1 .. 2 loop
          glob := elem12(e).glob(loc);
          vertex(glob,vx);
          put(io,vx(1),1,6,3);
          put(io,",");
          put(io,vx(2),1,6,3);
          put(io,",");
          put(io,vx(3),1,6,3);
          put(io," ");
        end loop;
        new_line(io);
      end loop;

      put(io,"layer off Initial_12 ");
      new_line(io);
    end if;

    put(io,"-color");
    new_line(io);
    put(io,"white");
    new_line(io);

    put(io,"layer set 0 ");
    new_line(io);

    put(io,"zoom ext ext");
    new_line(io);

    close(io);
  end autocad;

  procedure output is

    use
      text_io,
      ada.integer_text_io,
      ada.long_float_text_io;

    i  : integer;
    vx : fmat3x1;
    io : file_type;

  begin
    create(io, out_file, "output.txt");

    if (elem12'last = 0) and (elem22'last = 0) and (elem32'last = 0) then
      put(io,"Minor Iterations  = ");
      put(io,minor,13);
      new_line(io);

      put(io,"Max Gradient Norm = ");
      put(io,maxnlag,2,6,3);
      new_line(io);
    else
      put(io,"Major Iterations           = ");
      put(io,major,13);
      new_line(io);

      put(io,"Last Minor Iterations      = ");
      put(io,minor,13);
      new_line(io);

      put(io,"Max Lagrange Gradient Norm = ");
      put(io,maxnlag,2,6,3);
      new_line(io);

      put(io,"Max Constraint Norm        = ");
      put(io,maxncon,2,6,3);
      new_line(io);
    end if;

    -- displacements
    new_line(io);
    new_line(io);
    put(io," Node           Displ X           Displ Y           Displ Z");
    new_line(io);

    new_line(io);
    for glob in 1 .. nn loop
      put(io,glob,5);
      for axis in 1 .. 3 loop
        put(io,"     ");
        i := index(glob,axis);
        if i > n then
          put(io,known(glob,axis),2,6,3);
        else
          put(io,x(i),2,6,3);
        end if;
      end loop;
      new_line(io);
    end loop;

    -- vertex location
    new_line(io);
    new_line(io);
    put(io," Node           Coord X           Coord Y           Coord Z");
    new_line(io);

    new_line(io);
    for glob in 1 .. nn loop
      vertex(glob,vx);
      put(io,glob,5);
      for axis in 1 .. 3 loop
        put(io,"     ");
        put(io,vx(axis),2,6,3);
      end loop;
      new_line(io);
    end loop;

    close(io);
  end output;

begin
  input;

  setup;

  if (elem12'last = 0) and (elem22'last = 0) and (elem32'last = 0) then
    qn.lbfgs(maxlist,n,maxminor,maxls,
             minor,
             maxnlag,lstol,
             f,
             x,g,
             optimal);
  else
    auglag;
  end if;

  autocad;

  output;
end minshape;
